/*
[-----------------------------------------------------------------------General Notes and Comments]
 Carl Wegman
 Sugar Grove, Illinois, U.S.A.
 Pachinko Sensor, Counter & Games Program
 2013 Version 11 Mega
 Page Set Up Landscape L 0.35, R 0.35 T 0.5, B 0.5
 
 [---------------------------------------------------------------------------Libraries & Variables]
 */
// Libraries Included
#include <Bounce.h>          // Include Bounce Library to debounce inputs (sensors)
#include <Keypad.h>          // Include Keypad Library for matrix keypad
#include <EEPROM.h>          // Include EEPROM Library to read / write high scores 
//#include <SoftwareSerial.h>  // Include Software Serial Library for display

//SoftwareSerial NSS( 32, 33 );  // Set pins for Display
// Input Pin 32 (RX) to (TX) on display back pack
// Input Pin 33 (TX) to (RX) on display back pack

// Arduino Pin Variables
byte ShotSensorPin = 2; // The Arduino pin number attached to the Shot sensor collector (white)
byte WinSensorPin = 3;  // The Arduino pin number attached to the Win sensor collector (white)

byte BellOnOffButtonPin = 34; // The Arduino pin number attached to the Bell On / Off Button
byte WinMicroSwitchPin = 35; // The Arduino pin number attached to the Win Microswitch
byte ShotLEDPin = 36; // The Arduino pin number attached to the Shot LED
byte WinLEDPin = 37;  // The Arduino pin number attached to the Win LED
byte BellButtonSwitchLEDPin = 38; // The Arduino pin number for the Bell Button indicator LED
byte BellRelayPin = 39; // The Arduino pin number attached to the Bell Relay
byte ServoRelayPin = 40; // The Arduino pin number attached to the Servo Relay
byte ServoRelayButtonPin = 41; // The Arduino pin number attached to the Servo Button
byte SoftwareSerialRXPin = 32; // The Arduino pin number used for Software Serial RX
// byte MicroSwitch2Pin = 42; // The Arduino pin number attached to the Microswitch 2
// byte MicroSwitch3Pin = 43; // The Arduino pin number attached to the Microswitch 3

// Instantiate a Bounce object with a 3 ms debounce - this needs to be tested and adjusted
Bounce bouncerBellButton = Bounce( BellOnOffButtonPin, 3 ); // 3 is debounce in milliseconds
Bounce bouncerWinSwitch = Bounce( WinMicroSwitchPin, 3 ); // 3 is debounce in milliseconds
Bounce bouncerServoButton = Bounce( ServoRelayButtonPin, 3 ); // 3 is debounce in milliseconds

// Keypad Variables
const byte ROWS = 4;          //four rows
const byte COLS = 4;          //three columns
char keys[ ROWS ][ COLS ] = { // This varies per keypad
  { 
    '1','2','3','A'                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       }  
  ,
  { 
    '4','5','6','B'                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       }  
  ,
  { 
    '7','8','9','C'                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       }  
  ,
  { 
    '*','0','#','D'                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       }
};

/*
 byte rowPins[ ROWS ] = {
 A7,A6,A5,A4 }; //connect to the row pinouts of the keypad
 byte colPins[ COLS ] = {
 A3,A2,A1,A0 }; //connect to the column pinouts of the keypad
 Keypad keypad = Keypad( makeKeymap ( keys ), rowPins, colPins, ROWS, COLS ); 
 
 Black Matrix Keypad
 Row 0 = keypad pin 1, Input A7; Row 1 = keypad pin 2, Input A6
 Row 2 = keypad pin 3, Input A5; Row 3 = keypad pin 4, Input A4
 Col 0 = keypad pin 5, Input A3; Col 1 = keypad pin 6, Input A2
 Col 2 = keypad pin 7, Input A1; Col 3 = keypad pin 8, Input A0
 */
byte rowPins[ ROWS ] = {
  A7,A6,A5,A4 }; //connect to the row pinouts of the keypad
byte colPins[ COLS ] = {
  A3,A2,A1,A0 }; //connect to the column pinouts of the keypad
Keypad keypad = Keypad( makeKeymap ( keys ), rowPins, colPins, ROWS, COLS ); 

// Game Variables
int volatile Shots = 0; // Shots sensed by sensor
int PreviousShots = 0; 
int volatile Wins = 0;  // Wins sensed by sensor
int PreviousWins = 0;
byte GameSelection = 0;   // Game A = 1; Game B = 2; Game C = Repeat Last Game
long AGameTimeLength = 0; // Length of A Game Time in milliseconds
byte GameAInProgress = 0; // 0 = False, 1 = True
byte GameBInProgress = 0; // 0 = False, 1 = True
long GameAStartTime = 0;  // Used to calculate end of Game A
int BGameShotNumber = 0;  // Number of Shots for B Game Shot
byte WaitingForFirstShot = 0; // 0 = False, 1 = True
byte GameOver = 0; // 0 = False, 1 = True Used to show or hide Previous Game Option
int PreviousGameNumberOfPlayers = 0; // Store in EEPROM used for previous game selection
long PreviousGame = 0; // Store in EEPROM used for previous game selection
long PreviousAGameTimeLength = 0; // Store in EEPROM used for previous game selection
int PreviousBGameShotNumber; // Store in EEPROM used for previous game selection
int BallsWonPerJackpot = 15; // Used to compute score
unsigned long OldTime = 0;   // Used for counter
long ShotLEDTimerStartTime = 0; // Used to turn On and Off Shot LED
long WinLEDTimerStartTime = 0;  // Used to turn On and Off Win LED & Bell Relay
long ServoStartTime = 0; // Used to turn On and Off Servo Relay
int ServoDuration = 500; // Servo duration in milliseconds

// LED & Bell Variables
byte ShotLEDEnabled = 0; // 0 = disabled; 1 = enabled
byte WinLEDEnabled = 0; // 0 = disabled; 1 = enabled
byte WinSwitchEnabled = 0;  // 0 = disabled; 1 = enabled
byte ServoRelayEnabled = 0;  // 0 = disabled; 1 = enabled
byte BellRelayEnabled = 0; // 0 = disabled; 1 = enabled
byte BellButtonSwitch = 0; // 0 = disabled; 1 = enabled
byte BellButtonSwitchLED = 0; // 0 = disabled; 1 = enabled
int ShotLEDDuration = 250; // Shot LED duration in milliseconds
int WinLEDDuration = 1000; // Shot LED duration in milliseconds
int BellDuration = 1100; // Shot LED duration in milliseconds

// EEPROM Variables
int EEPROMLocations[] = {  // First EEPROM Location for stored vaules
  0, 1, 14, 27, 40, 53, 57, 60, 65, 67, 69, 71, 76, 81, 86, 90, 103,116, 129,
  142, 155, 160, 165, 170, 175, 180, 185, 190, 195, 200, 253, 266, 279, 292,
  305, 318, 320, 323, 325, 327, 340, 353, 366, 379, 384, 389, 394, 399, 401, 403, 405, 407
};
int EEPROMLength[] = {    // length of value stored
  1, 12, 12, 12, 12, 3, 2, 4, 1, 1, 1, 4, 4, 4, 2, 12, 12,
  12, 12, 12, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 12, 12, 12, 12,
  12, 1, 1, 1, 1, 12, 12, 12, 12, 4, 4, 4, 4, 1, 1, 1, 1, 4
};

// Menu Variables
char intro[] = "Press * for Games\nD for Options";
char gameintro[] = "\nPress Game Letter!\nA. Timer Game\nB. Shot Counter Game";
char gameintroC[] = "C. Replay Last Game";
char players[] = "Press 1 to 4 Players";
char GameA[] = "Game A\nTimer Game\n"; // \n = move cursor to beginning of next line
char GameB[] = "Game B\nShot Counter Game\n";
char GetName[] = "NAME? <= 12 letters\nA=Accept C=Clear\nB=Back D=Next\n#=Next Name Letter\n"; 
char GameLength[] = "Enter Game Length\nIn Seconds 30 to 600#=Accept C=Clear\n";
char GameLine3[] = "\e[3;1H~SHOT\e[3;6H~WON\e[3;11H~TIME\e[3;16H~SCORE";
//   \e[3;9H~ = Move cursor to line 3 position 9
char BallNumber[] = "Enter # of Shots\n 10 to 1000\n#=Accept C=Clear\n";
char ConfirmDefault[] = "\n# = OK; C = CHANGE\n";
byte ShowMenu = 0; // Used to show menu again after a game is completed
char Initials[] = {
  'A','B','C','D','E','F','G','H','I','J','K','L','M',
  'N','O','P','Q','R','S','T','U','V','W','X','Y','Z','\0'};
byte DelayMultiplierEnabled = 1;
int DelayMultiplierWait = 5;

// Player Variables
byte NumberOfPlayers = 0;      // Used to indicate number of players from 1 to 4
byte CurrentPlayerNumber = 0; // Current Player Number from 1 to 4
int PlayerScoreArray[] = {
  -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};
int ScoreRankArray2 [] = {
  0, 0, 0, 0,
  0, 0, 0, 0,
  0, 0, 0, 0,
  0, 0, 0, 0,
};
int ScoreRankArray[] = {
  0,0,0,0,1,2,3,4
};
byte PlayersNames [] = { // Used to store Players Names
  80, 108, 97, 121, 101, 114, 32, 49, 32, 32, 32, 32, //  0 - 11 = "Player 1    "
  80, 108, 97, 121, 101, 114, 32, 50, 32, 32, 32, 32, // 12 - 23 = "Player 2    "
  80, 108, 97, 121, 101, 114, 32, 51, 32, 32, 32, 32, // 24 - 35 = "Player 3    "
  80, 108, 97, 121, 101, 114, 32, 52, 32, 32, 32, 32  // 36 - 47 = "Player 4    "
};

/*
[---------------------------------------------------------------------END of Libraries & Variables]
 
 [-------------------------------------------------------------------------START of Function setup]
 */

void setup() {
  // setup code here, to run once:
  // Local Variables ( EEPROMread0, EEPROMread8, EEPROMread9, EEPROMread10
  // x, EEPROMread49, EEPROMread47, EEPROMread48 )

  pinMode( SoftwareSerialRXPin, INPUT ); // sets the SoftwareSerialRXPin pin as input
  //delete above pinMode if using Software Serial
  Serial1.begin( 19200 );    // Begin Serial1 for display
  // NSS.begin( 19200 );    // Begin Software Serial for display
  delay ( 2000 );        // Wait for display to start
  Serial1.print ( "\f" );

  pinMode( ShotSensorPin, INPUT ); // sets the Shot Sensor pin as input
  attachInterrupt( 0, ShotwasSensed, FALLING ); 

  byte EEPROMread49 = EEPROM.read( EEPROMLocations[ 49 ] );
  if ( EEPROMread49 == 1 ) { // check EEPROM to see if Win Microswitch enabled is true;
    // default = 0 = disabled.  This is a value that is not represented in ASCII
    pinMode( WinMicroSwitchPin, INPUT ); // sets the Win Microswitch pin as input for Wins
    digitalWrite( WinMicroSwitchPin, HIGH); // turns on internal pull up resistor
    WinSwitchEnabled = 1; // 1 = enabled
  }
  else if ( EEPROMread49 == 0 ) { // check EEPROM to see if Win Microswitch enabled is false;
    // default = 0 = disabled.  This is a value that is not represented in ASCII
    pinMode( WinSensorPin, INPUT ); // sets the Win Sensor pin as input for Wins
    attachInterrupt( 1, WinwasSensed, FALLING );
  }

  byte EEPROMread0 = EEPROM.read( EEPROMLocations[ 0 ] );
  if ( EEPROMread0 != 1 ) { // check EEPROM to see if populated;
    // default = 255 = not populated. This is a value that is not represented in ASCII
    WriteDefaults ( 1 ); // Calls WriteDefaults Function
  }
  byte EEPROMread8 = EEPROM.read( EEPROMLocations[ 8 ] );
  if ( EEPROMread8 == 1 ) { // check EEPROM to see if Shot LED enabled is true;
    // default = 1 = Enabled.  This is a value that is not represented in ASCII
    ShotLEDEnabled = 1;
    pinMode( ShotLEDPin, OUTPUT ); // sets the Shot LED Test pin as output
  }
  byte EEPROMread9 = EEPROM.read( EEPROMLocations[ 9 ] );
  if ( EEPROMread9 == 1 ) { // check EEPROM to see if Win LED enabled is true;
    // default = 1 = Enabled.  This is a value that is not represented in ASCII
    WinLEDEnabled = 1;
    pinMode( WinLEDPin, OUTPUT ); // sets the Win LED Test pin as output
  }
  byte EEPROMread10 = EEPROM.read( EEPROMLocations[ 10 ] );
  if ( EEPROMread10 == 1 ) { // check EEPROM to see if Bell enabled is true;
    // default = 1 = Enabled.  This is a value that is not represented in ASCII
    BellRelayEnabled = 1;
    pinMode( BellRelayPin, OUTPUT ); // sets the Bell Relay pin as output
    digitalWrite( BellRelayPin, HIGH );
    // sets the Bell Relay off
  }
  byte EEPROMread47 = EEPROM.read( EEPROMLocations[ 47 ] );
  if ( EEPROMread47 == 1 ) { // check EEPROM to see if Bell Button enabled is true;
    // default = 1 = Enabled.  This is a value that is not represented in ASCII
    BellButtonSwitch = 1;
    pinMode( BellOnOffButtonPin, INPUT ); // sets the Bell On Off Button pin as input    
  }
  byte EEPROMread48 = EEPROM.read( EEPROMLocations[ 48 ] );
  if ( EEPROMread48 == 1 ) { // check EEPROM to see if Bell Button LED enabled is true;
    // default = 1 = Enabled.  This is a value that is not represented in ASCII
    BellButtonSwitchLED = 1;
    pinMode( BellButtonSwitchLEDPin, OUTPUT ); // sets the Bell Button LED pin as output
    if ( BellRelayEnabled == 1 ) {
      // Turn on Bell Button LED Indicator & Relay
      if ( BellButtonSwitchLED == 1 ) {
        digitalWrite( BellButtonSwitchLEDPin, HIGH );
        digitalWrite( BellRelayPin, LOW );
      }
    }
  }  
  byte EEPROMread50 = EEPROM.read( EEPROMLocations[ 50 ] );
  if ( EEPROMread50 == 1 ) { // check EEPROM to see if Servo Relay enabled is true;
    // default = 0 = disabled.  This is a value that is not represented in ASCII
    ServoRelayEnabled = 1;
    pinMode( ServoRelayPin, OUTPUT ); // sets the Servo Relay pin as output
    digitalWrite( ServoRelayPin, HIGH );  // sets the Servo off
    pinMode( ServoRelayButtonPin, INPUT ); // sets the Servo Button pin as input    
  }  

  // Load More Default Values with ReadValue Function
  AGameTimeLength = ReadValue ( 5 ) * 1000 ; // Convert to milliseconds
  BGameShotNumber = ReadValue ( 7 );
  ShotLEDDuration = ReadValue ( 11 );
  WinLEDDuration = ReadValue ( 12 );
  BellDuration = ReadValue ( 13 );
  BallsWonPerJackpot = ReadValue ( 14 );
  DelayMultiplierWait = ReadValue ( 36 );
  ServoDuration = ReadValue ( 51 );


  // Load Players Names
  for ( int x = 0; x < 12; x++ ) {
    PlayersNames [ x ] = EEPROM.read( EEPROMLocations[ 1 ] + x ); // Player 1
    PlayersNames [ x + 12 ] = EEPROM.read( EEPROMLocations[ 2 ] + x ); // Player 2
    PlayersNames [ x + 24 ] = EEPROM.read( EEPROMLocations[ 3 ] + x ); // Player 3
    PlayersNames [ x + 36 ] = EEPROM.read( EEPROMLocations[ 4 ] + x ); // Player 4
  }

  Serial1.print( '\f' );   // \f = Clear screen
  Serial1.print ( intro ); // "Press * for Games\nD for Defaults"
}

/*
[----------------------------------------------------------------------------END of Function setup]
 
 [-----------------------------------------------------------START of Function loop - MAIN PROGRAM]
 */

void loop() {
  // main code here, to run repeatedly:
  // Local Variables ( getkey,  )

  CheckSensors(); // Call CheckSensors Function

  char getkey = GetKeypadInput ( 1 );
  if ( getkey == '*' ) {      // * was pressed on keypad
    GameMenu ();              // Call GameMenu Function
    Serial1.print( '\f' );        // \f = Clear screen
    Serial1.print ( intro );      // "Press * for Games\nD for Defaults"
  }
  else if ( getkey == 'D' ) { // D was pressed on keypad
    ShowOptions ();           // Call ShowOptions Function
    Serial1.print( '\f' );        // \f = Clear screen
    Serial1.print ( intro );      // "Press * for Games\nD for Defaults"

  }
}
/*
[--------------------------------------------------------------END of Function loop - MAIN PROGRAM]
 
 [------------------------------------------------------------------START of Function CheckSensors]
 */

void CheckSensors () {
  // LOOP FUNCTION Check Sensors for Shots and Wins
  // local variables used in this loop ( ShotSensor, WinSensor, )

  boolean WinSensor = false;
  boolean ShotSensor = false;

  // Update the debouncers
  if ( WinSwitchEnabled == 1 ) { // 1 = enabled
    bouncerWinSwitch.update (); // Use Win Microswitch to detect Wins
  }
  if ( BellButtonSwitch == 1 ) { // 1 = enabled
    bouncerBellButton.update ();
  }
  if ( ServoRelayEnabled == 1 ) { // 1 = enabled
    bouncerServoButton.update ();
  }

  // Get the update values
  boolean ServoButtonPressed = bouncerServoButton.fallingEdge ();
  // The fallingEdge method is true for one scan after the de-bounced input goes from on-to-off
  if ( ServoButtonPressed == true ) {
    // Turn on Servo for Servo duration    
    ServoStartTime = millis ();  // Start Shot LED Timer
    digitalWrite( ServoRelayPin, LOW );   // sets the Servo on  
  }
  if ( millis () - ServoStartTime >= ServoDuration ) { // check Servo Timer
    digitalWrite( ServoRelayPin, HIGH );  // sets the Servo off
  }
  if ( PreviousShots != Shots ){
    ShotSensor = true;
  }
  // The fallingEdge method is true for one scan after the de-bounced input goes from on-to-off
  if ( WinSwitchEnabled == 1 ) { // 1 = enabled
    WinSensor = bouncerWinSwitch.risingEdge (); // Use Win Microswitch to detect Wins
    // The risingEdge method is true for one scan after the de-bounced input goes from off-to-on
    if ( WinSensor == true ) {
      Wins ++;
    }
  }
  else if ( WinSwitchEnabled == 0 ) { // 0 = disabled
    if ( PreviousWins != Wins ){
      WinSensor = true;
    }
  }
  if ( BellButtonSwitch == 1 ) { // 1 = enabled
    boolean BellOnOff = bouncerBellButton.fallingEdge ();
    // The fallingEdge method is true for one scan after the de-bounced input goes from on-to-off
    if ( BellOnOff == true ){  // occurs when Bell On Off Button is pressed
      // byte BellRelayEnabled = 0; // 0 = disabled; 1 = enabled
      if ( BellRelayEnabled == 0 ) {
        BellRelayEnabled = 1; // 1 = enabled
        // Turn on "bell on" LED & Relay
        if ( BellButtonSwitchLED == 1 ) {
          digitalWrite( BellButtonSwitchLEDPin, HIGH );
          digitalWrite( BellRelayPin, LOW );
        }   
      }
      else if ( BellRelayEnabled == 1 ) {
        BellRelayEnabled = 0; // 0 = disabled;
        // Turn off "bell on" LED & Relay
        digitalWrite( BellButtonSwitchLEDPin, LOW );
        digitalWrite( BellRelayPin, HIGH );  
      }    
    }
  }
  if ( ShotSensor == true ){ // occurs when Shot is sensed
    PreviousShots = Shots;   // Increments Shots Variable = increase by 1
    UpdateDisplay ( 1 );     // Update Display for Shot sensed
  }
  if ( WinSensor == true ){  // occurs when Win is sensed
    PreviousWins = Wins;     // Increments Wins Variable = increase by 1
    UpdateDisplay ( 2 );     // Update Display for Win sensed
  }
  if ( ShotSensor != true && WinSensor != true ){
    UpdateDisplay ( 3 );     // Update Display if no Shots or Wins are sensed
  }
  // Verify if LEDs or bell are still in display cycle and turn off
  if ( ShotLEDEnabled == 1 ) { // 1 = Enabled
    if ( millis () - ShotLEDTimerStartTime >= ShotLEDDuration ) { // check Shot LED Timer
      digitalWrite( ShotLEDPin, LOW );
      // sets the Shot LED off (RED)
    }
  }
  if ( WinLEDEnabled == 1 ) { // 1 = Enabled
    if ( millis () - WinLEDTimerStartTime >= WinLEDDuration ) { //check Win LED Timer
      digitalWrite( WinLEDPin, LOW );
      // sets the Win LED off (RED)
    }
  }
  /*
  if ( BellRelayEnabled == 1 ) { // 1 = Enabled
   if ( millis () - WinLEDTimerStartTime >= BellDuration ) { // check Bell Relay Timer
   digitalWrite( BellRelayPin, HIGH );
   // sets the Bell Relay off
   }
   }
   */
  return; // Return to Main Loop
}

/*
[---------------------------------------------------------------------END of Function CheckSensors]
 
 [-----------------------------------------------------------------START of Function UpdateDisplay]
 */
void UpdateDisplay ( byte UpdateD ) {
  // Local Variables ( Seconds, Minutes, Remainder, SCORE )
  // Passed Values ( UpdateD = 1 = Shot Sensed, UpdateD = 2 = Win Sensed,
  // UpdateD = 3 = Other than Shot or Win )

  switch ( UpdateD ) {
  case 1: // Shot sensed
    if ( ShotLEDEnabled == 1 ) {          // 1 = Enabled; Turn on Shot LED for Shot LED duration
      ShotLEDTimerStartTime = millis ();  // Start Shot LED Timer
      digitalWrite( ShotLEDPin, HIGH );   // sets the Shot LED on (RED)
      if ( millis () - ShotLEDTimerStartTime >= ShotLEDDuration ) { // check Shot LED Timer
        digitalWrite( ShotLEDPin, LOW );  // sets the Shot LED off (RED)
      }
    }
    break;
  case 2: // Win sensed
    if ( WinLEDEnabled == 1 ) {         // 1 = Enabled; Turn on Win LED for Win LED duration
      WinLEDTimerStartTime = millis (); // Start Win LED Timer
      digitalWrite( WinLEDPin, HIGH );  // sets the Win LED on
      if ( millis () - WinLEDTimerStartTime >= WinLEDDuration) { // check Win LED Timer
        digitalWrite( WinLEDPin, LOW ); // sets the Win LED off
      }
    }
    // Turn on Bell for Bell duration
    /*
    if ( BellRelayEnabled == 1 ) {          // 1 = Enabled; Turn on Bell Relay for Bell duration
     digitalWrite( BellRelayPin, LOW );   // sets the Bell Relay on
     if ( millis () - WinLEDTimerStartTime >= BellDuration ) { // check Bell Relay Timer
     digitalWrite( BellRelayPin, HIGH );  // sets the Bell Relay off
     }
     }
     */
    break;
  case 3:                      // Shot or Win not sensed this cycle
    Serial1.print ( GameLine3 );   // "\e[3;1H~SHOT\e[3;6H~WON\e[3;11H~TIME\e[3;16H~SCORE"
    // \e[3;1H~ = Move cursor to line 3 position 1
    Serial1.print ( "\e[4;1H~" );  // Move cursor to line 4 position 1
    Serial1.print ( Shots );
    Serial1.print ( "\e[4;6H~" );  // Move cursor to line 4 position 6
    if (  Wins * BallsWonPerJackpot < 9999 ) {
      Serial1.print ( Wins * BallsWonPerJackpot );
    }
    else if (  Wins * BallsWonPerJackpot >= 9999 ) {
      Serial1.print ( "9999" );
    }
    Serial1.print ( "\e[4;11H~" ); // Move cursor to line 4 position 11
    if ( GameAInProgress == 0 && WaitingForFirstShot == 0 ){ // Regular Counter
      long Seconds = ( millis() - OldTime ) / 1000;
      if ( Seconds == 5998 ) {
        OldTime = millis(); // Reset Counter to zero at 99:98
        return;
      }
      int Minutes = 0;
      if ( Seconds >= 60 ) {
        Minutes = Seconds / 60;
      }
      int Remainder = Seconds - Minutes * 60;
      if ( Remainder < 10 ) {
        Serial1.print ( Minutes );
        Serial1.print ( ":0" );
        Serial1.print ( Remainder );
      }
      else if ( Remainder >= 10 ) {
        Serial1.print ( Minutes );
        Serial1.print ( ":" );
        Serial1.print ( Remainder );
      }
    }
    else if ( GameAInProgress == 1 && WaitingForFirstShot == 0 ){ // Count down game timer
      long DisplayTimeRemaining = AGameTimeLength + GameAStartTime - millis ();
      long Seconds = DisplayTimeRemaining / 1000;
      int Minutes = 0;
      if ( Seconds >= 60 ) {
        Minutes = Seconds / 60;
      }
      int Remainder = Seconds - ( Minutes * 60 );
      if ( Remainder < 10 && DisplayTimeRemaining >= 0 ) {
        Serial1.print ( Minutes );
        Serial1.print ( ":0" );
        Serial1.print ( Remainder );
      }
      else if ( Remainder >= 10 && DisplayTimeRemaining >= 0 ) {
        Serial1.print ( Minutes );
        Serial1.print ( ":" );
        Serial1.print ( Remainder );
      }
      else if ( DisplayTimeRemaining < 0 ) {
        GameAInProgress = 0;
      }
    }
    else if ( WaitingForFirstShot == 1 ) { // Waiting for first shot timer displays 0:00
      Serial1.print ( "0:00" );
    }
    int SCORE = Wins * BallsWonPerJackpot - Shots;
    Serial1.print ( "\e[2I~" ); // Turn off Auto Scroll
    if ( SCORE < -999 ) {
      SCORE = -999;
      Serial1.print ( "\e[4;16H~" ); // Move cursor to line 4 position 16
      Serial1.print ( " " ); // 1 space
      Serial1.print ( SCORE );
    }
    else if ( SCORE < -99 ) {      
      Serial1.print ("\e[4;16H~");   // Move cursor to line 4 position 16
      Serial1.print ( " " ); // 1 space
      Serial1.print (SCORE);
    }
    else if ( SCORE < -9 ) {      
      Serial1.print ("\e[4;16H~");   // Move cursor to line 4 position 16
      Serial1.print ( "  " ); // 2 space
      Serial1.print (SCORE);
    }
    else if ( SCORE < 0 ) {
      Serial1.print ( "\e[4;16H~" ); // Move cursor to line 4 position 16
      Serial1.print ( "   " ); // 4 spaces
      Serial1.print ( SCORE );
    }
    else if ( SCORE < 10 ) {
      Serial1.print ("\e[4;16H~");   // Move cursor to line 4 position 16
      Serial1.print ("    "); // 4 spaces
      Serial1.print (SCORE);
    }
    else if ( SCORE < 100 ) {
      Serial1.print ( "\e[4;16H~" ); // Move cursor to line 4 position 16
      Serial1.print ( "   " ); // 3 spaces
      Serial1.print ( SCORE );
    }
    else if ( SCORE < 1000 ) {
      Serial1.print ("\e[4;16H~");   // Move cursor to line 4 position 16
      Serial1.print ( "  " ); // 2 space      
      Serial1.print (SCORE);
    }
    else if ( SCORE < 10000 ) {
      Serial1.print ( "\e[4;16H~" ); // Move cursor to line 4 position 16
      Serial1.print ( " " ); // 1 space
      Serial1.print ( SCORE );
    }
    else if ( SCORE >= 10000 ) {
      Serial1.print ( "\e[4;16H~" ); // Move cursor to line 4 position 16
      SCORE = 9999;
      Serial1.print ( " " ); // 1 space
      Serial1.print ( SCORE );
    }
    Serial1.print ( "\e[1I~" ); // Turn on Auto Scroll
    break;
  }
  return;
}

/*
[--------------------------------------------------------------------END of Function UpdateDisplay]
 
 [-----------------------------------------------------------------START of Function WriteDefaults]
 */
void WriteDefaults ( byte WRDF ) {
  // Local Variables ( wd, defaultvalue1 through defaultvalue36 ( some are re-used ))

  // Passed Variable WRDF ( WRDF = 1 = Full Restore; WRDF = 2 = Restore High Scores - Timer Game
  // WRDF = 3 = Restore High Scores - Counter Game )

  if ( WRDF < 2 ) {
    //Default Value 0 - EEPROM Populated = 1
    byte defaultvalue0 [] = { 
      1
    };
    for ( int wd = 0; wd < EEPROMLength[ 0 ]; wd++ ) {
      EEPROM.write (( EEPROMLocations[ 0 ] + wd ), defaultvalue0[ wd ] );
    }

    // Default Value 1 - Player 1 Name = "Player 1    " in ASCII
    byte defaultvalue1 [] = { 
      80, 108, 97, 121, 101, 114, 32, 49, 32, 32, 32, 32
    };
    for ( int wd = 0; wd < EEPROMLength[ 1 ]; wd++ ) {    
      EEPROM.write (( EEPROMLocations[ 1 ] + wd ), defaultvalue1[ wd ] );
    }

    // Default Value 2 - Player 2 Name = "Player 2    " in ASCII
    byte defaultvalue2 [] = { 
      80, 108, 97, 121, 101, 114, 32, 50, 32, 32, 32, 32
    };
    for ( int wd = 0; wd < EEPROMLength[ 2 ]; wd++ ) {
      EEPROM.write (( EEPROMLocations[ 2 ] + wd ), defaultvalue2[ wd ] );
    }

    // Default Value 3 - Player 3 Name = "Player 3    " in ASCII
    byte defaultvalue3 [] = { 
      80, 108, 97, 121, 101, 114, 32, 51, 32, 32, 32, 32
    };
    for ( int wd = 0; wd < EEPROMLength[ 3 ]; wd++ ) {
      EEPROM.write (( EEPROMLocations[ 3 ] + wd ), defaultvalue3[ wd ] );
    }

    // Default Value 4 - Player 4 Name = "Player 4    " in ASCII
    byte defaultvalue4 [] = { 
      80, 108, 97, 121, 101, 114, 32, 52, 32, 32, 32, 32
    };
    for ( int wd = 0; wd < EEPROMLength[ 4 ]; wd++ ) {
      EEPROM.write (( EEPROMLocations[ 4 ] + wd ), defaultvalue4[ wd ] );
    }

    // Default Value 5 - Default time (seconds) for Game A Timer Game three digits
    // = "60 " in ASCII
    byte defaultvalue5 [] = { 
      54, 48, 32
    };
    for ( int wd = 0; wd < EEPROMLength[5]; wd++ ) {
      EEPROM.write (( EEPROMLocations[ 5 ] + wd ), defaultvalue5[ wd ] );
    }

    // Default Value 6 - Default Contrast setting 2 digits from 0 to 62 on DigiOle display
    // = "20" in ASCII
    byte defaultvalue6 [] = { 
      50,48
    };
    for ( int wd = 0; wd < EEPROMLength[ 6 ]; wd++ ) {
      EEPROM.write (( EEPROMLocations[ 6 ] + wd ), defaultvalue6[ wd ] );
    }

    // Default Value 7 - Default number of shots for Game B Shot Counter Game four digits
    // = "100 " in ASCII
    byte defaultvalue7 [] = { 
      49, 48, 48, 32
    };
    for ( int wd = 0; wd < EEPROMLength[ 7 ]; wd++ ) {
      EEPROM.write (( EEPROMLocations[ 7 ] + wd ), defaultvalue7[ wd ] );
    }

    // Default Value 8 - Shot LED Enabled = 1 = Enabled
    byte defaultvalue8 [] = { 
      1
    };
    for ( int wd = 0; wd < EEPROMLength[ 8 ]; wd++ ) {
      EEPROM.write (( EEPROMLocations[ 8 ] + wd ), defaultvalue8[ wd ] );
    }

    // Default Value 9 - Win LED Enabled = 1 = Enabled
    byte defaultvalue9 [] = { 
      1
    };
    for ( int wd = 0; wd < EEPROMLength[ 9 ]; wd++ ) {
      EEPROM.write (( EEPROMLocations[ 9 ] + wd ), defaultvalue9[ wd ] );
    }

    // Default Value 10 - Bell Relay Enabled = 1 = Enabled
    byte defaultvalue10 [] = { 
      1
    };
    for ( int wd = 0; wd < EEPROMLength[ 10 ]; wd++ ) {
      EEPROM.write (( EEPROMLocations[ 10 ] + wd ), defaultvalue10[ wd ] );
    }

    // Default Value 11 - Shot LED Duration (milliseconds) four digits
    // = "250 " in ASCII
    byte defaultvalue11 [] = { 
      50, 53, 48, 32
    };
    for ( int wd = 0; wd < EEPROMLength[ 11 ]; wd++ ) {
      EEPROM.write (( EEPROMLocations[ 11 ] + wd ), defaultvalue11[ wd ] );
    }

    // Default Value 12 - Win LED Duration (milliseconds) four digits
    // = "1000" in ASCII
    byte defaultvalue12 [] = { 
      49, 48, 48, 48
    };
    for ( int wd = 0; wd < EEPROMLength[ 12 ]; wd++ ) {
      EEPROM.write (( EEPROMLocations[ 12 ] + wd ), defaultvalue12[ wd ] );
    }

    // Default Value 13 - Bell Duration (milliseconds) four digits
    // = "1100" in ASCII
    byte defaultvalue13 [] = { 
      49, 49, 48, 48
    };
    for ( int wd = 0; wd < EEPROMLength[ 13 ]; wd++ ) {
      EEPROM.write (( EEPROMLocations[ 13 ] + wd ), defaultvalue13[ wd ] );
    }

    // Default Value 14 - Balls per Win two digits = "15" in ASCII
    byte defaultvalue14 [] = { 
      49, 53
    };
    for ( int wd = 0; wd < EEPROMLength[ 14 ]; wd++ ) {
      EEPROM.write (( EEPROMLocations[ 14 ] + wd ), defaultvalue14[ wd ] );
    }
  }

  if ( WRDF == 1 || WRDF == 2 || WRDF == 3 ) {
    // Default Value 15 & 30 - Player Name High Score 1 = "Carl W      " in ASCII
    byte defaultvalue15 [] = { 
      67, 97, 114, 108, 32, 87, 32, 32, 32, 32, 32, 32
    };
    for ( int wd = 0; wd < EEPROMLength[ 15 ]; wd++ ) {
      if ( WRDF == 1 || WRDF == 2 ) {
        EEPROM.write (( EEPROMLocations[ 15 ] + wd ), defaultvalue15[ wd ] );
      }
      if ( WRDF == 1 || WRDF == 3 ) {
        EEPROM.write (( EEPROMLocations[ 30 ] + wd ), defaultvalue15[ wd ] );
      }
    }

    // Default Value 16 & 31 - Player Name High Score 2 = "Kyle W      " in ASCII
    byte defaultvalue16 [] = { 
      75, 121, 108, 101, 32, 87, 32, 32, 32, 32, 32, 32
    };
    for ( int wd = 0; wd < EEPROMLength[ 16 ]; wd++ ) {
      if ( WRDF == 1 || WRDF == 2 ) {
        EEPROM.write (( EEPROMLocations[ 16 ] + wd ), defaultvalue16[ wd ] );
      }
      if ( WRDF == 1 || WRDF == 3 ) {
        EEPROM.write (( EEPROMLocations[ 31 ] + wd ), defaultvalue16[ wd ] );
      }
    }

    // Default Value 17 & 32 - Player Name High Score 3 = "Daniel W    " in ASCII
    byte defaultvalue17 [] = { 
      68, 97, 110, 105, 101, 108, 32, 87, 32, 32, 32, 32
    };
    for ( int wd = 0; wd < EEPROMLength[ 17 ]; wd++ ) {
      if ( WRDF == 1 || WRDF == 2 ) {
        EEPROM.write (( EEPROMLocations[ 17 ] + wd ), defaultvalue17[ wd ] );
        if ( WRDF == 1 || WRDF == 3 ) {
        }
        EEPROM.write (( EEPROMLocations[ 32 ] + wd ), defaultvalue17[ wd ] );
      }
    }

    // Default Value 18 & 33 - Player Name High Score 4 = "Russell W   " in ASCII
    byte defaultvalue18 [] = { 
      82, 117, 115, 115, 101, 108, 108, 32, 87, 32, 32, 32
    };
    for ( int wd = 0; wd < EEPROMLength[ 18 ]; wd++ ) {
      if ( WRDF == 1 || WRDF == 2 ) {
        EEPROM.write (( EEPROMLocations[ 18 ] + wd ), defaultvalue18[ wd ] );
      }
      if ( WRDF == 1 || WRDF == 3 ) {
        EEPROM.write (( EEPROMLocations[ 33 ] + wd ), defaultvalue18[ wd ] );    
      }
    }

    // Default Value 19 & 34 - Player Name High Score 5 = "Mary W      " in ASCII
    byte defaultvalue19 [] = { 
      77, 97, 114, 121, 32, 87, 32, 32, 32, 32, 32, 32
    };
    for ( int wd = 0; wd < EEPROMLength[ 19 ]; wd++ ) {
      if ( WRDF == 1 || WRDF == 2 ) {
        EEPROM.write (( EEPROMLocations[ 19 ] + wd ), defaultvalue19[ wd ] );
      }
      if ( WRDF == 1 || WRDF == 3 ) {
        EEPROM.write (( EEPROMLocations[ 34 ] + wd ), defaultvalue19[ wd ] );    
      }
    }

    // Default Value 20 & 25 - Default High Score 1 - 4 Digits
    // = "150 " in ASCII
    byte defaultvalue20 [] = { 
      49, 53, 48, 32
    };
    for ( int wd = 0; wd < EEPROMLength[ 20 ]; wd++ ) {
      if (WRDF == 1 || WRDF == 2 ) {
        EEPROM.write (( EEPROMLocations[ 20 ]  + wd ), defaultvalue20[ wd ] );
      }
      if ( WRDF == 1 || WRDF == 3 ) {
        EEPROM.write (( EEPROMLocations[ 25 ] + wd ), defaultvalue20[ wd ] );
      }
    }

    // Default Value 21 & 26 - Default High Score 2 - 4 Digits
    // = "140 " in ASCII
    byte defaultvalue21 [] = { 
      49, 52, 48, 32
    };
    for ( int wd = 0; wd < EEPROMLength[ 21 ]; wd++ ) {
      if ( WRDF == 1 || WRDF == 2 ) {
        EEPROM.write (( EEPROMLocations[ 21 ] + wd ), defaultvalue21[ wd ] );
      }
      if ( WRDF == 1 || WRDF == 3 ) {
        EEPROM.write (( EEPROMLocations[ 26 ] + wd ), defaultvalue21[ wd ] );
      }
    }

    // Default Value 22 & 27 - Default High Score 3 - 4 Digits
    // = "130 " in ASCII
    byte defaultvalue22 [] = { 
      49, 51, 48, 32
    };
    for ( int wd = 0; wd < EEPROMLength[ 22 ]; wd++ ) {
      if ( WRDF == 1 || WRDF == 2 ) {
        EEPROM.write (( EEPROMLocations[ 22 ] + wd ), defaultvalue22[ wd ] );
      }
      if ( WRDF == 1 || WRDF == 3 ) {
        EEPROM.write (( EEPROMLocations[ 27 ] + wd ), defaultvalue22[ wd ] );
      }
    }

    // Default Value 23 & 28 - Default High Score 4 - 4 Digits
    // = "120 " in ASCII
    byte defaultvalue23 [] = { 
      49, 50, 48, 32
    };
    for ( int wd = 0; wd < EEPROMLength[ 23 ]; wd++ ) {
      if ( WRDF == 1 || WRDF == 2 ) {
        EEPROM.write (( EEPROMLocations[ 23 ] + wd ), defaultvalue23[ wd ] );
      }
      if ( WRDF == 1 || WRDF == 3 ) {
        EEPROM.write (( EEPROMLocations[ 28 ] + wd ), defaultvalue23[ wd ] );
      }
    }

    // Default Value 24 & 29 - Default High Score 5 - 4 Digits
    // = "110 " in ASCII
    byte defaultvalue24 [] = { 
      49, 49, 48, 32
    };
    for ( int wd = 0; wd < EEPROMLength[ 24 ]; wd++ ) {
      if ( WRDF == 1 || WRDF == 2 ) {
        EEPROM.write (( EEPROMLocations[ 24 ] + wd ), defaultvalue24[ wd ] );
      }
      if ( WRDF == 1 || WRDF == 3 ) {
        EEPROM.write (( EEPROMLocations[ 29 ] + wd ), defaultvalue24[ wd ] );
      }
    }
  }

  if ( WRDF < 2 ) {
    // Default Value 35 - Delay Multiplier Enabled = 1 = Enabled
    byte defaultvalue35 [] = { 
      1
    };
    for ( int wd = 0; wd < EEPROMLength[ 35 ]; wd++ ) {
      EEPROM.write (( EEPROMLocations[ 35 ] + wd ), defaultvalue35[ wd ] );
    }

    // Default Value 36 - Delay Multiplier ( Times ) - one digits = "5" in ASCII
    byte defaultvalue36 [] = {  
      53
    };
    for ( int wd = 0; wd < EEPROMLength[ 36 ]; wd++ ) {
      EEPROM.write (( EEPROMLocations[ 36 ] + wd ), defaultvalue36[ wd ] );
    }

    // Default Value 47 - Bell On Off Momentary Contact Button Enabled = 1 = Enabled
    byte defaultvalue47 [] = { 
      1
    };
    for ( int wd = 0; wd < EEPROMLength[ 47 ]; wd++ ) {
      EEPROM.write (( EEPROMLocations[ 47 ] + wd ), defaultvalue47[ wd ] );
    }

    // Default Value 48 - Bell Button LED Enabled = 1 = Enabled
    byte defaultvalue48 [] = { 
      1
    };
    for ( int wd = 0; wd < EEPROMLength[ 48 ]; wd++ ) {
      EEPROM.write (( EEPROMLocations[ 48 ] + wd ), defaultvalue48[ wd ] );
    }

    // Default Value 49 - Win Microswitch Enabled = 0 = disabled
    byte defaultvalue49 [] = { 
      0
    };
    for ( int wd = 0; wd < EEPROMLength[ 49 ]; wd++ ) {
      EEPROM.write (( EEPROMLocations[ 49 ] + wd ), defaultvalue49[ wd ] );
    }

    // Default Value 50 - Servo Relay Enabled = 0 = disabled
    byte defaultvalue50 [] = { 
      0
    };
    for ( int wd = 0; wd < EEPROMLength[ 50 ]; wd++ ) {
      EEPROM.write (( EEPROMLocations[ 50 ] + wd ), defaultvalue50[ wd ] );
    } 

    // Default Value 51 - Servo Duration (milliseconds) four digits
    // = "500 " in ASCII
    byte defaultvalue51 [] = { 
      53, 48, 48, 32
    };
    for ( int wd = 0; wd < EEPROMLength[ 51 ]; wd++ ) {
      EEPROM.write (( EEPROMLocations[ 51 ] + wd ), defaultvalue51[ wd ] );
    }    
  }
  return;
}
/*
[--------------------------------------------------------------------END of Function WriteDefaults]
 
 [----------------------------------------------------------------START of Function GetKeypadInput]
 */
char GetKeypadInput ( byte gki ){
  // Local Variables ( keyi )
  // Passed Values ( gki )

  if ( gki == 1 ) {
    char keyi = keypad.getKey();
    if (keyi){
    }
    return keyi ;
  }
}

/*
[-------------------------------------------------------------------END of Function GetKeypadInput]
 
 [----------------------------------------------------------------START of Function ChangeDefaults]
 */

void ChangeDefaults () {
  // Local Variables ( cd1, WaitForConfirm1, getkey, value8, through value50 )


  Serial1.print ( "\f" ); // Clear Screen

  //Stored in EEPROM Default 1 = Player 1 Name = "Player 1    "
  Serial1.print ( "Player 1 Name = \n" );
  for ( int cd1 = 0; cd1 < EEPROMLength[ 1 ]; cd1++ ) {
    Serial1.print ( char ( EEPROM.read(( EEPROMLocations[ 1 ] + cd1 ))));    
  }
  Serial1.print ( ConfirmDefault );        // "\n# = OK; C = CHANGE\n"
  byte WaitForConfirm1 = 0;
  while ( WaitForConfirm1 == 0 ) {
    char getkey = GetKeypadInput ( 1 );
    if ( getkey == '#' ) {            // don't change default
      WaitForConfirm1 = 1;
    }
    else if ( getkey == 'C' ) {       // Change default
      Serial1.print ( "\f" );
      Serial1.print ( "Change Default 1\n" );
      delay ( 600 * DelayMultiplierWait );
      Serial1.print ( "\f" );
      ChangeName ( 1 , 1 );               // Call ChangeName Function
      WaitForConfirm1 = 1;
    }
  }

  //Stored in EEPROM Default 2 = Player 2 Name = "Player 2    "
  Serial1.print ( "Player 2 Name = \n" );
  for ( int cd1 = 0; cd1 < EEPROMLength[ 2 ]; cd1++ ) {
    Serial1.print ( char ( EEPROM.read(( EEPROMLocations[ 2 ] + cd1 ))));
  }
  Serial1.print ( ConfirmDefault ); // "\n# = OK; C = CHANGE\n"
  WaitForConfirm1 = 0;
  while ( WaitForConfirm1 == 0 ) {
    char getkey = GetKeypadInput ( 1 );
    if ( getkey == '#' ) { // don't change default
      WaitForConfirm1 = 1;
    }
    else if ( getkey == 'C' ) {
      Serial1.print ( "\f" );
      Serial1.print ( "Change Default 2\n" );
      delay ( 600 * DelayMultiplierWait );
      Serial1.print ( "\f" );
      ChangeName ( 2, 1 );
      WaitForConfirm1 = 1;
    }
  }

  //Stored in EEPROM Default 3 = Player 3 Name = "Player 3    " 
  Serial1.print ( "Player 3 Name = \n" );
  for ( int cd1 = 0; cd1 < EEPROMLength[ 3 ]; cd1++ ) {
    Serial1.print ( char ( EEPROM.read(( EEPROMLocations[ 3 ] + cd1 ))));
  }
  Serial1.print ( ConfirmDefault ); // "\n# = OK; C = CHANGE\n"
  WaitForConfirm1 = 0;
  while ( WaitForConfirm1 == 0 ) {
    char getkey = GetKeypadInput ( 1 );
    if ( getkey == '#' ) { // don't change default
      WaitForConfirm1 = 1;
    }
    else if ( getkey == 'C' ) {
      Serial1.print ( "\f" );
      Serial1.print ( "Change Default 3\n" );
      delay ( 600 * DelayMultiplierWait );
      Serial1.print ( "\f" );
      ChangeName ( 3, 1 );
      WaitForConfirm1 = 1;
    }
  }

  //Stored in EEPROM Default 4 = Player 2 Name = "Player 4    "
  Serial1.print ( "Player 4 Name = \n" );
  for ( int cd1 = 0; cd1 < EEPROMLength[ 4 ]; cd1++ ) {
    Serial1.print ( char ( EEPROM.read(( EEPROMLocations[ 4 ] + cd1 ))));
  }
  Serial1.print ( ConfirmDefault ); // "\n# = OK; C = CHANGE\n"
  WaitForConfirm1 = 0;
  while ( WaitForConfirm1 == 0 ) {
    char getkey = GetKeypadInput ( 1 );
    if ( getkey == '#' ) { // don't change default
      WaitForConfirm1 = 1;
    }
    else if ( getkey == 'C' ) {
      Serial1.print ( "\f" );
      Serial1.print ( "Change Default 4\n" );
      delay ( 600 * DelayMultiplierWait );
      Serial1.print ( "\f" );
      ChangeName ( 4, 1 );
      WaitForConfirm1 = 1;
    }
  }

  // Stored in EEPROM Default 5
  // Default time (seconds) for Game A Timer Game - three digits
  // Default = "60 " or in ASCII = 54, 48, 32
  Serial1.print ( "Game Time Length =\n" );
  for ( int cd1 = 0; cd1 < EEPROMLength[ 5 ]; cd1++ ) {
    Serial1.print ( char ( EEPROM.read(( EEPROMLocations[ 5 ] + cd1 ))));
  } 
  Serial1.print ( " seconds" );
  Serial1.print ( ConfirmDefault ); // "\n# = OK; C = CHANGE\n"
  WaitForConfirm1 = 0;
  while ( WaitForConfirm1 == 0 ) {
    char getkey = GetKeypadInput ( 1 );
    if ( getkey == '#' ) { // don't change default
      WaitForConfirm1 = 1;
    }
    else if ( getkey == 'C' ) {
      Serial1.print ( "\f" );
      long OldAGameTimeLength = AGameTimeLength; // Save Old value
      GetAGameTimeLength ( 2 );
      // EEPROM value is changed inside function GetAGameTimeLength ( 2 )
      // Restore High Scores and High Score Player Names
      if ( OldAGameTimeLength != AGameTimeLength ) { // Check for change in value
        WriteDefaults ( 2 );
      }
      WaitForConfirm1 = 1;
      Serial1.print ( "\f" );
    }
  }

  // Stored in EEPROM Default 7
  // Default number of shots for Game B Shot Counter Game - four digits
  // Default = "100 " or in ASCII = 49, 48, 48, 32
  Serial1.print ( "# of Shots =\n" );
  for ( int cd1 = 0; cd1 < EEPROMLength[ 7 ]; cd1++ ) {
    Serial1.print ( char ( EEPROM.read(( EEPROMLocations[ 7 ] + cd1 ))));
  } 
  Serial1.print ( " shots" );
  Serial1.print ( ConfirmDefault ); // "\n# = OK; C = CHANGE\n"  
  WaitForConfirm1 = 0;
  while ( WaitForConfirm1 == 0 ) {
    char getkey = GetKeypadInput ( 1 );
    if ( getkey == '#' ) { // don't change default
      WaitForConfirm1 = 1;
    }
    else if ( getkey == 'C' ) {
      Serial1.print ( "\f" );
      int OldBGameShotNumber = BGameShotNumber; // Save Old value
      GetBGameShotNumber ( 2 );
      // EEPROM value is changed inside function GetBGameShotNumber ( 2 )
      if ( OldBGameShotNumber != BGameShotNumber ) { // Check for change in value
        WriteDefaults ( 3 );
      }
      WaitForConfirm1 = 1;
      Serial1.print ( "\f" );           
    }
  }

  //Stored in EEPROM Default 8 Shot LEDs Enabled 
  Serial1.print ( "Shot LED Enabled =\n" );
  int value8 =  EEPROM.read( EEPROMLocations[ 8 ] );
  if ( value8 == 0 ) {
    Serial1.print ( "disabled" ); 
  }
  else if ( value8 == 1 ) {
    Serial1.print ( "Enabled" ); 
  } 
  Serial1.print ( ConfirmDefault ); // "\n# = OK; C = CHANGE\n"
  WaitForConfirm1 = 0;
  while ( WaitForConfirm1 == 0 ) {
    char getkey = GetKeypadInput ( 1 );
    if ( getkey == '#' ) { // don't change default
      WaitForConfirm1 = 1;
    }
    else if ( getkey == 'C' ) {
      Serial1.print ( "\f" );
      Serial1.print ( "Changed Default 8\nPower Off Required!" );
      delay ( 600 * DelayMultiplierWait );
      Serial1.print ( "\f" );
      // change value here
      if ( value8 == 1 ) { // change from enabled to disabled
        EEPROM.write ( EEPROMLocations [ 8 ], 0 );
        ShotLEDEnabled = 0;
      }
      else if ( value8 == 0 ) { // change from disabled to enabled
        EEPROM.write ( EEPROMLocations [ 8 ], 1 );
        ShotLEDEnabled = 1;
      }
      WaitForConfirm1 = 1;
    }
  }

  //Stored in EEPROM Default 11 - Shot LED Duration (milliseconds) - four digits
  // = "1000" in ASCII = 
  if ( ShotLEDEnabled == 1 ) { // only allowed if Shot LED is enabled
    Serial1.print ( "Shot LED Duration =\n" );
    for ( int cd1 = 0; cd1 < EEPROMLength[ 11 ]; cd1++ ) {
      Serial1.print ( char ( EEPROM.read(( EEPROMLocations[ 11 ] + cd1 ))));
    } 
    Serial1.print ( " milliseconds" );
    Serial1.print (ConfirmDefault); // "\n# = OK; C = CHANGE\n"
    WaitForConfirm1 = 0;
    while ( WaitForConfirm1 == 0 ) {
      char getkey = GetKeypadInput ( 1 );
      if ( getkey == '#' ) { // don't change default
        WaitForConfirm1 = 1;
      }
      else if ( getkey == 'C' ) {
        Serial1.print ( "\f" );
        // change value here
        ValidateDigits ( 11, 4 ); // Default Value 11, 4 Digits
        // EEPROM value is changed in function ValidateDigits
        ShotLEDDuration = ReadValue ( 11 ); // Update value with change
        WaitForConfirm1 = 1;
      }
    }
  }

  //Stored in EEPROM Default 9 Win LEDs Enabled 
  Serial1.print ( "Win LED Enabled =\n" );
  int value9 =  EEPROM.read( EEPROMLocations[ 9 ] );
  if ( value9 == 0 ) {
    Serial1.print ( "disabled" ); 
  }
  else if ( value9 == 1 ) {
    Serial1.print ( "Enabled" ); 
  } 
  Serial1.print (ConfirmDefault); // "\n# = OK; C = CHANGE\n"
  WaitForConfirm1 = 0;
  while ( WaitForConfirm1 == 0 ) {
    char getkey = GetKeypadInput ( 1 );
    if ( getkey == '#' ) { // don't change default
      WaitForConfirm1 = 1;
    }
    else if ( getkey == 'C' ) {
      Serial1.print ( "\f" );
      Serial1.print ( "Changed Default 9\nPower Off Required!" );
      delay ( 600 * DelayMultiplierWait );
      Serial1.print ( "\f" );
      // change value here
      if ( value9 == 1 ) { // change from enabled to disabled
        EEPROM.write ( EEPROMLocations [ 9 ], 0 );
        WinLEDEnabled = 0;
      }
      else if ( value9 == 0 ) { // change from disabled to enabled
        EEPROM.write ( EEPROMLocations [ 9 ], 1 );
        WinLEDEnabled = 1; 
      }
      WaitForConfirm1 = 1;
    }
  }

  //Stored in EEPROM Default 12 - Win LED Duration (milliseconds) - four digits
  if ( WinLEDEnabled == 1 ) { // only allowed if Win LED is enabled
    Serial1.print ( "Win LED Duration =\n" );
    for ( int cd1 = 0; cd1 < EEPROMLength[ 12 ]; cd1++ ) {
      Serial1.print ( char ( EEPROM.read(( EEPROMLocations[ 12 ] + cd1 ))));
    } 
    Serial1.print ( " milliseconds" );
    Serial1.print (ConfirmDefault); // "\n# = OK; C = CHANGE\n"
    WaitForConfirm1 = 0;
    while ( WaitForConfirm1 == 0 ) {
      char getkey = GetKeypadInput ( 1 );
      if ( getkey == '#' ) { // don't change default
        WaitForConfirm1 = 1;
      }
      else if ( getkey == 'C' ) {
        Serial1.print ( "\f" );
        // change value here
        ValidateDigits ( 12, 4 ); // Default Value 12, 4 Digits
        // EEPROM value is changed in function ValidateDigits
        WinLEDDuration = ReadValue ( 12 ); // Update value with change
        WaitForConfirm1 = 1;
      }
    }
  }

  //Stored in EEPROM Default 10 Bell Relay Enabled 
  Serial1.print ( "Bell Enabled =\n" );
  int value10 =  EEPROM.read( EEPROMLocations[ 10 ] );
  if ( value10 == 0 ) {
    Serial1.print ( "disabled" ); 
  }
  else if ( value10 == 1 ) {
    Serial1.print ( "Enabled" ); 
  } 
  Serial1.print (ConfirmDefault); // "\n# = OK; C = CHANGE\n"
  WaitForConfirm1 = 0;
  while ( WaitForConfirm1 == 0 ) {
    char getkey = GetKeypadInput (1);
    if ( getkey == '#' ) { // don't change default
      WaitForConfirm1 = 1;
    }
    else if ( getkey == 'C' ) {
      Serial1.print ( "\f" );
      Serial1.print ( "Changed Default 10\nPower Off Required!" );
      delay ( 600 * DelayMultiplierWait );
      Serial1.print ( "\f" );
      // change value here
      if ( value10 == 1 ) { // change from enabled to disabled
        EEPROM.write ( EEPROMLocations [ 10 ], 0 );
        BellRelayEnabled = 0;
        digitalWrite( BellButtonSwitchLEDPin, LOW ); // Turn off "bell on" LED & Relay
        digitalWrite( BellRelayPin, HIGH );
      }
      else if ( value10 == 0 ) { // change from disabled to enabled
        EEPROM.write ( EEPROMLocations [ 10 ], 1 );
        BellRelayEnabled = 1;
      }
      WaitForConfirm1 = 1;
    }
  }

  //Stored in EEPROM Default 13 - Bell Duration (milliseconds) - four digits
  if ( BellRelayEnabled == 1 ) { // only allowed if Bell Relay is enabled
    Serial1.print ( "Bell Duration =\n" );
    for ( int cd1 = 0; cd1 < EEPROMLength[ 13 ]; cd1++ ) {
      Serial1.print ( char ( EEPROM.read(( EEPROMLocations[ 13 ] + cd1 ))));
    } 
    Serial1.print ( " milliseconds" );
    Serial1.print (ConfirmDefault); // "\n# = OK; C = CHANGE\n"
    WaitForConfirm1 = 0;
    while ( WaitForConfirm1 == 0 ) {
      char getkey = GetKeypadInput ( 1 );
      if ( getkey == '#' ) { // don't change default
        WaitForConfirm1 = 1;
      }
      else if ( getkey == 'C' ) {
        Serial1.print ( "\f" );
        // change value here
        ValidateDigits ( 13, 4 ); // Default Value 13, 4 Digits
        // EEPROM value is changed in function ValidateDigits
        BellDuration = ReadValue ( 13 ); // Update value with change
        WaitForConfirm1 = 1;
      }
    }
  }

  //Stored in EEPROM Default 14 - Balls Won per Jackpot - two digits  
  Serial1.print ( "Balls per Win =\n" );
  for ( int cd1 = 0; cd1 < EEPROMLength[ 14 ]; cd1++ ) {
    Serial1.print ( char ( EEPROM.read(( EEPROMLocations[ 14 ] + cd1 ))));
  } 
  Serial1.print ( " pachinko balls" );
  Serial1.print (ConfirmDefault); // "\n# = OK; C = CHANGE\n"
  WaitForConfirm1 = 0;
  while ( WaitForConfirm1 == 0 ) {
    char getkey = GetKeypadInput ( 1 );
    if ( getkey == '#' ) { // don't change default
      WaitForConfirm1 = 1;
    }
    else if ( getkey == 'C' ) {
      Serial1.print ( "\f" );
      // change value here
      ValidateDigits ( 14, 2 ); // Default Value 14, 2 Digits
      // EEPROM value is changed in function ValidateDigits
      BallsWonPerJackpot = ReadValue ( 14 ); // Update value with change
      WaitForConfirm1 = 1;
    }
  }

  //Stored in EEPROM Default 35 Delay Multiplier Enabled 
  Serial1.print ( "Delay Multiplier =\n" );
  int value35 =  EEPROM.read( EEPROMLocations[ 35 ] );
  if ( value35 == 0 ) {
    Serial1.print ( "disabled" );
    // Restore Default Value 36 - Delay Multiplier ( Times )
    // - one digit = "5" in ASCII
    byte defaultvalue36 [] = {  
      53
    };
    for ( int wd = 0; wd < EEPROMLength[ 36 ]; wd++ ) {
      EEPROM.write (( EEPROMLocations[ 36 ] + wd ), defaultvalue36[ wd ] );
    }
    DelayMultiplierWait = 5;    
  }
  else if ( value35 == 1 ) {
    Serial1.print ( "Enabled" ); 
  } 
  Serial1.print ( ConfirmDefault ); // "\n# = OK; C = CHANGE\n"
  WaitForConfirm1 = 0;
  while ( WaitForConfirm1 == 0 ) {
    char getkey = GetKeypadInput ( 1 );
    if ( getkey == '#' ) { // don't change default
      WaitForConfirm1 = 1;
    }
    else if ( getkey == 'C' ) {
      Serial1.print ( "\f" );
      Serial1.print ( "Changed Default 35\nPower Off Required!" );
      delay ( 600 * DelayMultiplierWait );
      Serial1.print ( "\f" );
      // change value here
      if ( value35 == 1 ) { // change from enabled to disabled
        EEPROM.write ( EEPROMLocations [ 35 ], 0 );
        DelayMultiplierEnabled = 0;
      }
      else if ( value35 == 0 ) { // change from disabled to enabled
        EEPROM.write ( EEPROMLocations [ 35 ], 1 );
        DelayMultiplierEnabled = 1;
      }
      WaitForConfirm1 = 1;
    }
  }

  if ( DelayMultiplierEnabled == 1 ) {
    //Stored in EEPROM Default 36 - Delay Multiplier Wait Minutes - two digits  
    Serial1.print ( "Delay Multiplier =\n" );
    for ( int cd1 = 0; cd1 < EEPROMLength[ 36 ]; cd1++ ) {
      Serial1.print ( char ( EEPROM.read(( EEPROMLocations[ 36 ] + cd1 ))));
    } 
    Serial1.print ( " times" );
    Serial1.print (ConfirmDefault); // "\n# = OK; C = CHANGE\n"
    WaitForConfirm1 = 0;
    while ( WaitForConfirm1 == 0 ) {
      char getkey = GetKeypadInput ( 1 );
      if ( getkey == '#' ) { // don't change default
        WaitForConfirm1 = 1;
      }
      else if ( getkey == 'C' ) {
        Serial1.print ( "\f" );
        // change value here
        ValidateDigits ( 36, 2 ); // Default Value 36, 2 Digits
        // EEPROM value is changed in function ValidateDigits
        DelayMultiplierWait = ReadValue ( 36 ); // Update value with change
        WaitForConfirm1 = 1;
      }
    }
  }  

  //Stored in EEPROM Default 47 Bell On Off Button Enabled 
  Serial1.print ( "Bell Button =\n" );
  int value47 =  EEPROM.read( EEPROMLocations[ 47 ] );
  if ( value47 == 0 ) {
    Serial1.print ( "disabled" ); 
  }
  else if ( value47 == 1 ) {
    Serial1.print ( "Enabled" ); 
  } 
  Serial1.print (ConfirmDefault); // "\n# = OK; C = CHANGE\n"
  WaitForConfirm1 = 0;
  while ( WaitForConfirm1 == 0 ) {
    char getkey = GetKeypadInput (1);
    if ( getkey == '#' ) { // don't change default
      WaitForConfirm1 = 1;
    }
    else if ( getkey == 'C' ) {
      Serial1.print ( "\f" );
      Serial1.print ( "Changed Default 47\nPower Off Required!" );
      delay ( 600 * DelayMultiplierWait );
      Serial1.print ( "\f" );
      // change value here
      if ( value47 == 1 ) { // change from enabled to disabled
        EEPROM.write ( EEPROMLocations [ 47 ], 0 );       
      }
      else if ( value47 == 0 ) { // change from disabled to enabled
        EEPROM.write ( EEPROMLocations [ 47 ], 1 );        
      }
      WaitForConfirm1 = 1;
    }
  } 

  //Stored in EEPROM Default 48 Bell On Off Button LED Enabled 
  Serial1.print ( "Bell Button LED =\n" );
  int value48 =  EEPROM.read( EEPROMLocations[ 48 ] );
  if ( value48 == 0 ) {
    Serial1.print ( "disabled" ); 
  }
  else if ( value48 == 1 ) {
    Serial1.print ( "Enabled" ); 
  } 
  Serial1.print (ConfirmDefault); // "\n# = OK; C = CHANGE\n"
  WaitForConfirm1 = 0;
  while ( WaitForConfirm1 == 0 ) {
    char getkey = GetKeypadInput (1);
    if ( getkey == '#' ) { // don't change default
      WaitForConfirm1 = 1;
    }
    else if ( getkey == 'C' ) {
      Serial1.print ( "\f" );
      Serial1.print ( "Changed Default 48\nPower Off Required!" );
      delay ( 600 * DelayMultiplierWait );
      Serial1.print ( "\f" );
      // change value here
      if ( value48 == 1 ) { // change from enabled to disabled
        EEPROM.write ( EEPROMLocations [ 48 ], 0 );       
      }
      else if ( value48 == 0 ) { // change from disabled to enabled
        EEPROM.write ( EEPROMLocations [ 48 ], 1 );        
      }
      WaitForConfirm1 = 1;
    }
  }  

  //Stored in EEPROM Default 49 Win Microswitch Enabled 
  Serial1.print ( "Win Microswitch =\n" );
  int value49 =  EEPROM.read( EEPROMLocations[ 49 ] );
  if ( value49 == 0 ) {
    Serial1.print ( "disabled" ); 
  }
  else if ( value49 == 1 ) {
    Serial1.print ( "Enabled" ); 
  } 
  Serial1.print (ConfirmDefault); // "\n# = OK; C = CHANGE\n"
  WaitForConfirm1 = 0;
  while ( WaitForConfirm1 == 0 ) {
    char getkey = GetKeypadInput (1);
    if ( getkey == '#' ) { // don't change default
      WaitForConfirm1 = 1;
    }
    else if ( getkey == 'C' ) {
      Serial1.print ( "\f" );
      Serial1.print ( "Changed Default 49\nPower Off Required!" );
      delay ( 600 * DelayMultiplierWait );
      Serial1.print ( "\f" );
      // change value here
      if ( value49 == 1 ) { // change from enabled to disabled
        EEPROM.write ( EEPROMLocations [ 49 ], 0 );       
      }
      else if ( value49 == 0 ) { // change from disabled to enabled
        EEPROM.write ( EEPROMLocations [ 49 ], 1 );        
      }
      WaitForConfirm1 = 1;
    }
  }

  //Stored in EEPROM Default 50 Servo Relay Enabled 
  Serial1.print ( "Servo Relay =\n" );
  int value50 =  EEPROM.read( EEPROMLocations[ 50 ] );
  if ( value50 == 0 ) {
    Serial1.print ( "disabled" ); 
  }
  else if ( value50 == 1 ) {
    Serial1.print ( "Enabled" ); 
  } 
  Serial1.print (ConfirmDefault); // "\n# = OK; C = CHANGE\n"
  WaitForConfirm1 = 0;
  while ( WaitForConfirm1 == 0 ) {
    char getkey = GetKeypadInput (1);
    if ( getkey == '#' ) { // don't change default
      WaitForConfirm1 = 1;
    }
    else if ( getkey == 'C' ) {
      Serial1.print ( "\f" );
      Serial1.print ( "Changed Default 50\nPower Off Required!" );
      delay ( 600 * DelayMultiplierWait );
      Serial1.print ( "\f" );
      // change value here
      if ( value50 == 1 ) { // change from enabled to disabled
        EEPROM.write ( EEPROMLocations [ 50 ], 0 );       
      }
      else if ( value50 == 0 ) { // change from disabled to enabled
        EEPROM.write ( EEPROMLocations [ 50 ], 1 );        
      }
      WaitForConfirm1 = 1;
    }
  } 

  //Stored in EEPROM Default 51 - Servo Duration (milliseconds) - four digits
  // = "500 " in ASCII = 
  if ( ServoRelayEnabled == 1 ) { // only allowed if Servo is enabled
    Serial1.print ( "Servo Duration =\n" );
    for ( int cd1 = 0; cd1 < EEPROMLength[ 51 ]; cd1++ ) {
      Serial1.print ( char ( EEPROM.read(( EEPROMLocations[ 51 ] + cd1 ))));
    } 
    Serial1.print ( " milliseconds" );
    Serial1.print (ConfirmDefault); // "\n# = OK; C = CHANGE\n"
    WaitForConfirm1 = 0;
    while ( WaitForConfirm1 == 0 ) {
      char getkey = GetKeypadInput ( 1 );
      if ( getkey == '#' ) { // don't change default
        WaitForConfirm1 = 1;
      }
      else if ( getkey == 'C' ) {
        Serial1.print ( "\f" );
        // change value here
        ValidateDigits ( 51, 4 ); // Default Value 11, 4 Digits
        // EEPROM value is changed in function ValidateDigits
        ServoDuration = ReadValue ( 51 ); // Update value with change
        WaitForConfirm1 = 1;
      }
    }
  }

  //Restore all Defaults to Original Settings 
  Serial1.print ( "\f" );
  Serial1.print ( "Use Initial Default?Will Erase ALL\nHigh Scores & Names" );
  Serial1.print ("\n# = NO; 1 = YES   ");
  WaitForConfirm1 = 0;
  while ( WaitForConfirm1 == 0 ) {
    char getkey = GetKeypadInput ( 1 );
    if ( getkey == '#' ) { // don't change default
      WaitForConfirm1 = 1;
    }
    else if ( getkey == '1' ) {
      Serial1.print ( "\f" );
      Serial1.print ( "Restored Initial\nSettings to Default\nPower Off Required!" );
      WriteDefaults ( 1 );
      delay ( 1000 * DelayMultiplierWait );
      Serial1.print ( "\f" );
      WaitForConfirm1 = 1;
    }
  }
  // Show Version Number
  Serial1.print ( "\f" );
  Serial1.print ( "   PACHINKO GAME\n   Version 11 Mega\n               2013" );
  delay ( 1000 * DelayMultiplierWait );
  Serial1.print ( "\f" );  
  // Reset Counter after changing defaults
  OldTime = millis();
  Shots = 0;
  Wins = 0;
  return;
}

/*
[-------------------------------------------------------------------END of Function ChangeDefaults]
 
 [----------------------------------------------------------------------START of Function GameMenu]
 */

void GameMenu () {
  // Local Variables ( x, getkey, tg, cg, )

  // Clear previous game PlayerScoreArray
  for ( int x = 1; x < 17; x++ ) {
    PlayerScoreArray [ x ] = 0;
  }
  Serial1.print ( "\f" );
  Serial1.print ( gameintro );   // "\nPress Game Letter!\nA. Timer Game\nB. Shot Counter Game"
  if ( GameOver == 1) {      // Used to show or hide Previous Game Option
    Serial1.print( gameintroC ); // "C. Replay Last Game"; if available
  }
  byte WaitForConfirm1 = 0;
  while ( WaitForConfirm1 == 0 ) {
    char getkey = GetKeypadInput ( 1 );
    if ( getkey == 'A' ) {
      GameSelection = 1; 
      GetAGameTimeLength ( 1 );
      Serial1.print( '\f' );
      Serial1.print ( players ); // "Press 1 to 4 Players"
      GetPlayers ();
      WaitForConfirm1 = 1;

      // The following occurs when Game A - Timer Game Loop is selected and 
      // cycles through each player
      for ( int tg = 0; tg < NumberOfPlayers; tg++ ) {
        CurrentPlayerNumber ++;        
        TimerGame ( CurrentPlayerNumber, NumberOfPlayers, AGameTimeLength, 0 );
        // Game A Timer Game Function passing variables
        // x = Current Player Number, y = Number of Players, z = A Game Time in milliseconds
      }
      CurrentPlayerNumber = 0;
    }   
    else if ( getkey == 'B' ) {
      // Game B - Counter Game Selected
      // Serial1.print('\f');
      GameSelection = 2; 
      GetBGameShotNumber ( 1 );
      Serial1.print ( '\f' );
      Serial1.print ( players ); // "Press 1 to 4 Players"
      GetPlayers ();
      WaitForConfirm1 = 1;

      // The following occurs when Game B - Shot Game Loop is selected and 
      // cycles through each player
      for ( int cg =0; cg < NumberOfPlayers; cg++ ) {
        CurrentPlayerNumber ++;
        CounterGame ( CurrentPlayerNumber, NumberOfPlayers, BGameShotNumber, 0 );
        // Game B Counter Game Function passing variables
        // x = Current Player Number, y = Number of Players, z = Number of Shots
      }
      CurrentPlayerNumber = 0;
    }
    else if ( getkey == 'C' && GameOver == 1 ) {
      GameSelection = PreviousGame;
      NumberOfPlayers = PreviousGameNumberOfPlayers; 
      AGameTimeLength = PreviousAGameTimeLength;
      BGameShotNumber = PreviousBGameShotNumber;
      WaitForConfirm1 = 1;

      // The following occurs when Game C - Previous Game Loop is selected and 
      // cycles through each player
      if ( GameSelection == 1 ) {
        for ( int tg = 0; tg < NumberOfPlayers; tg++ ) {
          CurrentPlayerNumber ++;        
          TimerGame ( CurrentPlayerNumber, NumberOfPlayers, AGameTimeLength, 1 );
          // Game A Timer Game Function passing variables
          // x = Current Player Number, y = Number of Players, z = A Game Time in milliseconds
        }
        CurrentPlayerNumber = 0;
      }
      else if ( GameSelection == 2 ) {
        for ( int cg =0; cg < NumberOfPlayers; cg++ ) {
          CurrentPlayerNumber ++;
          CounterGame ( CurrentPlayerNumber, NumberOfPlayers, BGameShotNumber, 1 );
          // Game B Counter Game Function passing variables
          // (x = Current Player Number, y = Number of Players, z = Number of Shots)
        }
        CurrentPlayerNumber = 0;
      }
    }
    else if ( getkey == '#' ) {
      return;
    }
  }  
  return;
}

/*
[-------------------------------------------------------------------------END of Function GameMenu]
 
 [--------------------------------------------------------------------START of Function ChangeName]
 */

void ChangeName( byte PNum, byte Store ) {
  // Local Variables ( OneTime, PlayerNameAccepted, PlayerName, PlayerNamearrayvalue,
  // charposition, keyi, )
  // Passed Values ( PNum = Player Number 1 through 4,
  // Store 0 = Do Not Store, Store 1 = Store in EEPROM )

  byte OneTime = 0;
  byte PlayerNameAccepted = 0;
  char PlayerName [] = {
    ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '                                                                                                                                                                                                                                                                                                                                                     };
  int PlayerNamearrayvalue = 0;
  Serial1.print ( "\f" );
  Serial1.print ( GetName );
  // "NAME? <= 12 letters\nA=Accept C=Clear\nB=Back D=Next\n#=Next Name Letter\n";
  int charposition = 0;
  Serial1.print ( Initials[ charposition ] );
  Serial1.print ( "\e[25m~" ); // Turn off cursor
  while ( PlayerNameAccepted == 0 ) {
    char keyi = keypad.getKey();
    if  ( keyi != NO_KEY ) { // Occurs when a keypad key is pressed
      if ( keyi == 'A' ) {
        PlayerNameAccepted = 1;
        PlayerName [ PlayerNamearrayvalue ] = Initials [ charposition ];
        if ( PlayerNamearrayvalue < 11 ) {
          PlayerNamearrayvalue++;
        }
        else if ( PlayerNamearrayvalue == 11 ) {
          PlayerNameAccepted = 1;
        }
        charposition = 0;
      }
      else if ( keyi == 'B') {
        // Back Selected
        if ( charposition != 0 ) {
          charposition --; // reverse position
        }
        else if ( charposition == 0 ) {
          charposition = 25; // reverse position to 0 ('Z')
        }
        Serial1.print ( "\e[1D~" ); // Move cursor left 1 space
        Serial1.print ( Initials[ charposition ] );
      }
      else if ( keyi == 'C' ) { // Clear Selected
        PlayerNamearrayvalue = 0;
        charposition = 0;
        Serial1.print ( "\e[4;1H~" ); // Move cursor to row 4 column 1
        Serial1.print ( "             " ); // Print 13 blank spaces
        Serial1.print ( "\e[13D~" ); // Move cursor left 13 spaces
        Serial1.print ( Initials[ charposition ] );
      }
      else if ( keyi == 'D' ) { // Next Selected
        if ( charposition != 25) {
          charposition ++; // advance position
        }
        else if ( charposition == 25) {
          charposition = 0; // advance position to 0 ('A')
        }
        Serial1.print ( "\e[1D~" ); // Move cursor left 1 space
        Serial1.print ( Initials[ charposition ] );
      }
      else if ( keyi == '#' ) { // Next Name Letter Selected
        PlayerName [ PlayerNamearrayvalue ] = Initials [ charposition ] ;
        if ( PlayerNamearrayvalue < 11 ) {
          PlayerNamearrayvalue++;
        }
        else if ( PlayerNamearrayvalue == 11 ) {
          PlayerNameAccepted = 1;
        }
        charposition = 0;
        Serial1.print ( Initials[ charposition ] );
      }
    }
  }
  if ( OneTime == 0 ) {
    Serial1.print ( "\f" );
    Serial1.print ( "\e[5m~" ); // Turn on cursor
    OneTime = 1;  
    //write to EEPROM now if Store = 1
    if ( Store == 1 ) { // Store Name in EEPROM
      for ( int x = 0; x < EEPROMLength [ PNum ]; x++) {
        EEPROM.write ( ( EEPROMLocations [ PNum ] + x ), PlayerName [ x ]);
      }
    }
    // if Store = 0 or Store = 1 Change Name Stored in PlayersNames Array
    for ( int x = 0; x < 12; x++ ) { // Change Name Stored in PlayersNames Array
      PlayersNames [ PNum * 12 - 12 + x ] = PlayerName [ x ];        
    }      
    Serial1.print ( "\f" );
  }
  return;
}

/*
[-----------------------------------------------------------------------END of Function ChangeName]
 
 [------------------------------------------------------------------------START-GetAGameTimeLength]
 */

void GetAGameTimeLength ( byte GAGTL ){  
  // Local Variables ( keyi, AGameTimeLengthFirstDigit, 
  // AGameTimeLengthSecondDigit, AGameTimeLengthThirdDigit, LengthSelectionProcess )
  // Passed Values ( GAGTL = 1 = Came from GameMenu ; GAGTL = 2 = Came from ChangeDefaults )

  // Check if Default Game Length is desired

  int LengthSelectionProcess = 0; // 0 = Started , 1 = In Process, 2 = Selected, 3 = Confirmed
  if ( GAGTL == 1 ) { 
    Serial1.print( '\f' ); // \f = Clear screen
    Serial1.print ( "Use Default Game\nTime Length?\n" );
    int AGTLRead = ReadValue ( 5 ); // Get default AGameTimeLength from EEPROM
    Serial1.print ( AGTLRead );
    Serial1.print ( " seconds?\n" );
    Serial1.print ( "#=Accept C=Change" );
    while ( LengthSelectionProcess != 3) {
      char keyi = keypad.getKey();
      if  ( keyi != NO_KEY ) {
        switch ( keyi ){
        case '#': // Accept Default
          //Valid A Game Time Length
          AGameTimeLength = AGTLRead * 1000; // Convert to milliseconds
          LengthSelectionProcess = 3; // 3 = Default
          return;
          break;
        case 'C': // Change
          LengthSelectionProcess = 3; // 3 = Continue with Change     
          break;      
        default: // If key is not C or #
          LengthSelectionProcess = 3; // 3 = Continue with Change      
          break;
        }
      }
    } 
  }
  // Determine Time Length Portion
  int AGameTimeLengthFirstDigit = -1;
  int AGameTimeLengthSecondDigit = -1;
  int AGameTimeLengthThirdDigit = -1;
  LengthSelectionProcess = 0; // 0 = Started , 1 = In Process, 2 = Selected, 3 = Confirmed
  if ( LengthSelectionProcess == 0 ) { // 0 = Started
    Serial1.print( '\f' ); // \f = Clear screen
    Serial1.print ( GameLength ); // "Enter Game Length\nIn Seconds 30 to 600#=Accept C=Clear\n"
    LengthSelectionProcess = 1; // 1 = In Process
  }
  while ( LengthSelectionProcess != 3) {
    char keyi = keypad.getKey();
    if  ( keyi != NO_KEY ) {
      switch ( keyi ){
      case '#': // Accept
        //Check for Valid A Game Time Length
        LengthSelectionProcess = 2; // 2 = Selected
        break;
      case 'C': // Cancel
        AGameTimeLength = 0;
        AGameTimeLengthFirstDigit = -1;
        AGameTimeLengthSecondDigit = -1;
        AGameTimeLengthThirdDigit = -1;
        Serial1.print('\f'); // \f = Clear screen // Delete Later
        Serial1.print ( GameLength ); // "Enter Game Length\nIn Seconds 30 to 600#=Accept C=Clear\n"
        LengthSelectionProcess = 1; // 1 = In Process
        break;
      case 'B':
        break;
      case 'D':
        break;
      case '*':
        break;
      case 'A':
        break;
      default: // If key is not A or C; B,D,*,#
        if ( AGameTimeLengthFirstDigit == -1 && keyi > 48 ) { // Don't allow first digit = '0')
          AGameTimeLengthFirstDigit = keyi - 48;
          Serial1.print ( AGameTimeLengthFirstDigit );
        } 
        else if ( AGameTimeLengthFirstDigit != -1 && AGameTimeLengthSecondDigit == -1 ) {
          AGameTimeLengthSecondDigit = keyi - 48;
          Serial1.print ( AGameTimeLengthSecondDigit );
        }
        else if ( AGameTimeLengthFirstDigit != -1 && AGameTimeLengthSecondDigit != -1  ) {
          AGameTimeLengthThirdDigit = keyi - 48;
          Serial1.print ( AGameTimeLengthThirdDigit );
          LengthSelectionProcess = 2; // 2 = Selected
        }
        break;
      }
    }
    if ( LengthSelectionProcess == 2 ) {
      if ( AGameTimeLengthThirdDigit == -1 ) {
        AGameTimeLength = AGameTimeLengthFirstDigit * 10 + AGameTimeLengthSecondDigit;
      }
      else if ( AGameTimeLengthThirdDigit != -1 ) {
        AGameTimeLength = AGameTimeLengthFirstDigit * 100 +
          AGameTimeLengthSecondDigit * 10 + AGameTimeLengthThirdDigit;
      }
      // Validate & Confirm
      if ( AGameTimeLength >= 30 && AGameTimeLength <=600 ) { // Change back to 30
        AGameTimeLength = AGameTimeLength * 1000; // Convert Seconds to Milliseconds
        LengthSelectionProcess = 3; // 3 = Confirmed
      }
      else if ( AGameTimeLength < 30 || AGameTimeLength > 600 ) { // 
        AGameTimeLength = 0;
        AGameTimeLengthFirstDigit = -1;
        AGameTimeLengthSecondDigit = -1;
        AGameTimeLengthThirdDigit = -1;
        Serial1.print('\f'); // \f = Clear screen
        Serial1.print ( GameLength ); // "Enter Game Length\nIn Seconds 30 to 600#=Accept C=Clear\n"
        LengthSelectionProcess = 1; // 1 = In Process
        Serial1.print ( AGameTimeLength );
      }
    } 
  }
  if ( GAGTL == 2 && LengthSelectionProcess == 3 ) { // Store Default 5
    //AGameTimeLength =  0;
    EEPROM.write ( EEPROMLocations[ 5 ], ( AGameTimeLengthFirstDigit + 48 ));
    // Convert Digit to ASCII
    EEPROM.write (( EEPROMLocations[ 5 ] + 1 ), ( AGameTimeLengthSecondDigit + 48 ));
    if ( AGameTimeLengthThirdDigit != -1 ) {
      EEPROM.write (( EEPROMLocations[ 5 ] + 2 ), ( AGameTimeLengthThirdDigit + 48 ));
    }
    else if ( AGameTimeLengthThirdDigit == -1 ) { // over write with space - ASCII 32 " "
      EEPROM.write (( EEPROMLocations[ 5 ] + 2 ), 32 );
    }
    Serial1.print('\f'); // \f = Clear screen
  }
  return;
}

/*
[------------------------------------------------------------------------END of GetAGameTimeLength]
 
 [---------------------------------------------------------------------START of GetBGameShotNumber]
 */
void GetBGameShotNumber ( byte GBGSN ){ 
  // Local Variables ( keyi, BGameShotNumberFirstDigit, 
  // BGameShotNumberSecondDigit, BGameShotNumberThirdDigit, BGameShotNumberFourthDigit
  // LengthSelectionProcess )
  // Passed Values ( GBGSN = 1 Not Stored in EEPROM, GBGSN = 2 Stored in EEPROM  )

  int LengthSelectionProcess = 0; // 0 = Started , 1 = In Process, 2 = Selected, 3 = Confirmed
  if ( GBGSN == 1 ) { // Came from Game Menu 
    Serial1.print( '\f' ); // \f = Clear screen
    Serial1.print ( "Use Default Number\nof Shots?\n" );
    int BGSNRead = ReadValue ( 7 ); // Get default BGameShotNumber from EEPROM
    Serial1.print ( BGSNRead );
    Serial1.print ( " Shots?\n" );
    Serial1.print ( "#=Accept C=Change" );
    while ( LengthSelectionProcess != 3) {
      char keyi = keypad.getKey();
      if  ( keyi != NO_KEY ) {
        switch ( keyi ){
        case '#': // Accept Default
          //Valid B Game Shot Number
          BGameShotNumber = BGSNRead; // Use Stored Default Shot Number
          LengthSelectionProcess = 3; // 3 = Use Default
          return;
          break;
        case 'C': // Change
          LengthSelectionProcess = 3; // 3 = Continue with change      
          break;      
        default: // If key is not C or #
          LengthSelectionProcess = 3; // 3 = Continue with change      
          break;
        }
      }
    } 
  }  
  int BGameShotNumberFirstDigit = -1;
  int BGameShotNumberSecondDigit = -1;
  int BGameShotNumberThirdDigit = -1;
  int BGameShotNumberFourthDigit = -1;

  LengthSelectionProcess = 0; // 0 = Started , 1 = In Process, 2 = Selected, 3 = Confirmed
  if ( LengthSelectionProcess == 0 ) { // 0 = Started
    Serial1.print( '\f' ); // \f = Clear screen
    Serial1.print ( BallNumber );  // "Enter # of Shots\n 10 to 1000\n#=Accept C=Clear\n"
    LengthSelectionProcess = 1; // 1 = In Process
  }
  while ( LengthSelectionProcess != 3 ) {
    char keyi = keypad.getKey();
    if  ( keyi != NO_KEY ) {
      switch ( keyi ){
      case '#': // Accept
        //Check for Valid B Game Number of Shots
        LengthSelectionProcess = 2; // 2 = Selected
        break;
      case 'C': // Cancel
        BGameShotNumber = 0;
        BGameShotNumberFirstDigit = -1;
        BGameShotNumberSecondDigit = -1;
        BGameShotNumberThirdDigit = -1;
        BGameShotNumberFourthDigit = -1;
        Serial1.print( '\f' ); // \f = Clear screen
        Serial1.print ( BallNumber ); // "Enter # of Shots\n 10 to 1000\n#=Accept C=Clear\n"
        LengthSelectionProcess = 1;  // 1 = In Process
        break;
      case 'B':
        break;
      case 'D':
        break;
      case '*':
        break;
      case 'A':
        break;
      default: // If key is not A or C; B,D,*,#
        if ( BGameShotNumberFirstDigit == -1 && keyi > 48 ) { // Don't allow first digit = '0'
          BGameShotNumberFirstDigit = keyi - 48; // Convert char to dec
          Serial1.print ( BGameShotNumberFirstDigit );
        } 
        else if ( BGameShotNumberFirstDigit != -1 && BGameShotNumberSecondDigit == -1 ) {
          BGameShotNumberSecondDigit = keyi - 48;  // Convert char to dec
          Serial1.print ( BGameShotNumberSecondDigit );
        }
        else if (BGameShotNumberFirstDigit != -1 && BGameShotNumberSecondDigit != -1
          && BGameShotNumberThirdDigit == -1  ) {
          BGameShotNumberThirdDigit = keyi - 48;   // 
          Serial1.print ( BGameShotNumberThirdDigit );
        }
        else if ( BGameShotNumberFirstDigit != -1 && BGameShotNumberSecondDigit != -1
          && BGameShotNumberThirdDigit != -1  ) {
          BGameShotNumberFourthDigit = keyi - 48;   // Convert char to dec
          Serial1.print ( BGameShotNumberFourthDigit );
          LengthSelectionProcess = 2; // 2 = Selected
        }
      }
    }
    if ( LengthSelectionProcess == 2 ) {
      if ( BGameShotNumberFourthDigit != -1 ) {
        BGameShotNumber = BGameShotNumberFirstDigit * 1000 + BGameShotNumberSecondDigit * 100 +
          BGameShotNumberThirdDigit * 10 + BGameShotNumberFourthDigit ;
      }
      else if ( BGameShotNumberFourthDigit == -1 && BGameShotNumberThirdDigit != -1) {
        BGameShotNumber = BGameShotNumberFirstDigit * 100 + BGameShotNumberSecondDigit * 10 +
          BGameShotNumberThirdDigit;
      }
      else if ( BGameShotNumberThirdDigit == -1 && BGameShotNumberSecondDigit != -1) {
        BGameShotNumber = BGameShotNumberFirstDigit * 10 + BGameShotNumberSecondDigit;
      }

      // Validate & Confirm
      if ( BGameShotNumber >= 10 && BGameShotNumber <=1000 ) { // Change back to 10
        LengthSelectionProcess = 3; // 3 = Confirmed
      }
      else if ( BGameShotNumber < 10 || BGameShotNumber > 1000 ) { 
        BGameShotNumber = 0;
        BGameShotNumberFirstDigit = -1;
        BGameShotNumberSecondDigit = -1;
        BGameShotNumberThirdDigit = -1;
        BGameShotNumberFourthDigit = -1;
        Serial1.print('\f'); // \f = Clear screen
        Serial1.print ( BallNumber ); // "Enter # of Shots\n 10 to 1000\n#=Accept C=Clear\n"
        LengthSelectionProcess = 1; // 1 = In Process
      }
    } 
  }
  if ( GBGSN == 2 && LengthSelectionProcess == 3 ) {
    //BGameShotNumber =  0;
    EEPROM.write ( EEPROMLocations[7], ( BGameShotNumberFirstDigit + 48 ));
    // Convert Digit to ASCII
    EEPROM.write (( EEPROMLocations[7] + 1 ), ( BGameShotNumberSecondDigit + 48 ));    
    if ( BGameShotNumberThirdDigit != -1 ) {
      EEPROM.write (( EEPROMLocations[7] + 2 ), ( BGameShotNumberThirdDigit + 48 ));
    }
    else if ( BGameShotNumberThirdDigit == -1 ) { // over write with space - ASCII 32 " "
      EEPROM.write (( EEPROMLocations[7] + 2 ), 32 );
    }
    if ( BGameShotNumberFourthDigit != -1 ) {
      EEPROM.write (( EEPROMLocations[7] + 3 ), ( BGameShotNumberFourthDigit + 48 ));
    }
    else if ( BGameShotNumberFourthDigit == -1 ) {  // over write with space - ASCII 32 " "
      EEPROM.write (( EEPROMLocations[7] + 3 ), 32 );
    }
    Serial1.print( '\f' ); // \f = Clear screen
  }
  return;
}

/*
[------------------------------------------------------------------------END of GetBGameShotNumber]
 
 [--------------------------------------------------------------------START of Function GetPlayers]
 */

void GetPlayers (){
  // Local Variables ( keyi, c, EEPROMValue )

  byte WaitForConfirm1 = 0;
  while ( WaitForConfirm1 == 0 ) {
    char keyi = GetKeypadInput(1);
    if  ( keyi != NO_KEY ) {
      switch ( keyi ){
      case '1':
        NumberOfPlayers = 1;
        WaitForConfirm1 = 1;
        break;  
      case '2':
        NumberOfPlayers = 2;
        WaitForConfirm1 = 1;
        break;
      case '3':
        NumberOfPlayers = 3;
        WaitForConfirm1 = 1;
        break;
      case '4':
        NumberOfPlayers = 4;
        WaitForConfirm1 = 1;
        break; 
      }
    }
  }
  return;
}

/*
[------------------------------------------------------------------------END of Function GetPlayers]
 
 [----------------------------------------------------------------------START of Function TimerGame]
 */

void TimerGame ( int CPN, int NoP, long AGTL, byte Replay ){ 
  // CPN = Current Player Number, NoP = Number of Players, AGTL = AGameTimeLength in milliseconds
  // Replay = 0 = Not a Replay of previous game; Replay = 1 = Previous Game Selections
  // Local Variables ( c )
  // Passed Values ( CPN, NoP, AGTL )

  // Check Player's Name
  int NameSelectionProcess = 0; // 0 = Started , 3 = Confirmed
  if ( Replay == 0 ) { // Not a Replay of Previous Game 
    Serial1.print( '\f' ); // \f = Clear screen
    Serial1.print ( "Use Default Player\nName = " );
    for ( int c = EEPROMLocations[ CPN ];
   c < EEPROMLocations[ CPN ] + EEPROMLength[ CPN ]; c++) {
        int EEPROMValue = EEPROM.read( c );
      if ( EEPROMValue != 255 ){
        Serial1.print ( char ( EEPROMValue ));   
      }  
    }    
    Serial1.print ( "\n#=Accept C=Change" );
    while ( NameSelectionProcess != 3) {
      char keyi = keypad.getKey();
      if  ( keyi != NO_KEY ) {
        switch ( keyi ){
        case '#': // Accept Default
          // Store in PlayersNames Array
          for ( int x = 0; x < 12; x++ ) { // Change Name Stored in PlayersNames Array
            PlayersNames [ CPN * 12 - 12 + x ] = EEPROM.read ( EEPROMLocations[ CPN ] + x );       
          } 
          NameSelectionProcess = 3; // 3 = Use Default Name
          break;
        case 'C': // Change
          ChangeName ( CPN, 0 ); // Do not store name in EEPROM
          NameSelectionProcess = 3; // 3 = Change Name      
          break;      
        default: // If key is not C or #
          NameSelectionProcess = 3; // 3 = Use Default Name      
          break;
        }
      }
    } 
  }
  // If Replay = 0 or Replay = 1  

  Serial1.print ( "\f" );
  // Check if name is "Player #"; l = 108 in ASCII
  if ( PlayersNames [ CPN * 12 -12 + 1 ] == 108 ) {    
    for ( int c = 0; c < 8; c++ ) {
      Serial1.print ( char ( PlayersNames [ CPN * 12 -12 + c ] ));
    }
  }
  else if ( PlayersNames [ CPN * 12 -12 + 1 ] != 108 ) {
    for ( int c = 0; c < 12; c++ ) {
      if ( PlayersNames [ CPN * 12 -12 + c ] != 32 ) {        
        Serial1.print ( char ( PlayersNames [ CPN * 12 -12 + c ] ));
      }      
    }
  }
  Serial1.print ( "'s Turn\n" );
  Serial1.print ( "Shot Starts Game" );
  Shots = 0;
  Wins = 0;
  if ( ServoRelayEnabled == 1 ) {
    // Turn on Servo for Servo duration    
    ServoStartTime = millis ();  // Start Shot LED Timer
    digitalWrite( ServoRelayPin, LOW );   // sets the Servo on  
  }
  while ( Shots == 0 ) {
    CheckSensors();
    WaitingForFirstShot = 1;
  }
  while ( Shots == 1 && GameAStartTime == 0) {
    CheckSensors ();
    WaitingForFirstShot = 0;
    GameAInProgress = 1;
    GameAStartTime = millis (); // Start Game Timer
  }
  while ( Shots != 0 && millis()- GameAStartTime <= AGTL) {
    CheckSensors ();    
  }
  if ( Shots != 0 && millis()- GameAStartTime > AGTL) {
    OldTime = millis();
    // Game Over via time
    GameOver = 1;
    // Verify if LEDs or bell are still in display cycle and turn off
    if ( ShotLEDEnabled == 1 ) { // 1 = Enabled    
      digitalWrite( ShotLEDPin, LOW );
      // sets the Shot LED off
    }
    if ( WinLEDEnabled == 1 ) { // 1 = Enabled    
      digitalWrite( WinLEDPin, LOW );
      // sets the Win LED off
    }
    /*
    if ( BellRelayEnabled == 1 ) { // 1 = Enabled    
     digitalWrite( BellRelayPin, HIGH );
     // sets the Bell Relay off
     }
     */
    // Record Score Timer Game
    int SCORE = Wins * BallsWonPerJackpot - Shots; // Compute Score
    if ( SCORE > 9999 ) {
      SCORE = 9999;
    }
    int AGameLength = AGameTimeLength / 1000; // Get A Game Time Length in seconds
    RecordScore ( GameSelection, SCORE, CPN, NoP, AGameLength );
    if ( CPN <= NoP ) {
      DisplayScreen ( 2, 4, GameSelection, SCORE, CPN, AGameLength );
      // Display Player Game Score for 4 Seconds * Delay Multiplier in Function Display Screen
    }
    Shots = 0; // Reset Shots Variable
    Wins = 0;  // Reset Wins Variable
    GameAStartTime = 0; // Reset
    if ( CPN == NoP ) { // if CPN = Current Player Number = NoP = Number of Players - RESET
      // All Players have played this game
      DisplayScreen ( 1, 5, GameSelection, SCORE, CPN, AGameLength );
      // Display Final Game Scores for 5 Seconds * Delay Multiplier in Function Display Screen 
      PreviousGame = 1;
      PreviousGameNumberOfPlayers = NoP;
      PreviousAGameTimeLength = AGTL;
      NumberOfPlayers = 0;
      GameSelection = 0;
      ShowMenu = 1;
      CurrentPlayerNumber = 0;
    }
  }
  return;
}

/*
[------------------------------------------------------------------------END of Function TimerGame]
 
 [-------------------------------------------------------------------START of Function CounterGame]
 */

void CounterGame ( int CPN, int NoP, int NoS, byte Replay ){
  // CPN = Current Player Number, NoP = Number of Players  NoS = Number of Shots
  // Replay = 0 = Not a Replay of previous game; Replay = 1 = Previous Game Selections
  // Local Variables ( c )
  // Passed Values ( CPN, NoP, NoS )

  // Check Player's Name
  int NameSelectionProcess = 0; // 0 = Started , 3 = Confirmed
  if ( Replay == 0 ) { // Not a Replay of Previous Game 
    Serial1.print( '\f' ); // \f = Clear screen
    Serial1.print ( "Use Default Player\nName = " );
    for ( int c = EEPROMLocations[ CPN ];
   c < EEPROMLocations[ CPN ] + EEPROMLength[ CPN ]; c++) {
        int EEPROMValue = EEPROM.read( c );
      if ( EEPROMValue != 255 ){
        Serial1.print ( char ( EEPROMValue ));   
      }  
    }    
    Serial1.print ( "\n #=Accept C=Change" );
    while ( NameSelectionProcess != 3) {
      char keyi = keypad.getKey();
      if  ( keyi != NO_KEY ) {
        switch ( keyi ){
        case '#': // Accept Default
          // Store in PlayersNames Array
          for ( int x = 0; x < 12; x++ ) { // Change Name Stored in PlayersNames Array
            PlayersNames [ CPN * 12 - 12 + x ] = EEPROM.read ( EEPROMLocations[ CPN ] + x );       
          }         
          NameSelectionProcess = 3; // 3 = Use Default Name
          break;
        case 'C': // Change
          ChangeName ( CPN, 0 ); // Do not store name in EEPROM
          NameSelectionProcess = 3; // 3 = Change Name      
          break;      
        default: // If key is not C or #
          NameSelectionProcess = 3; // 3 = Use Default Name      
          break;
        }
      }
    } 
  }
  // If Replay = 0 or Replay = 1  
  Serial1.print ( "\f" );
  // Check if name is "Player #"; l = 108 in ASCII
  if ( PlayersNames [ CPN * 12 -12 + 1 ] == 108 ) {
    for ( int c = 0; c < 8; c++ ) {
      Serial1.print ( char ( PlayersNames [ CPN * 12 -12 + c ] ));
    }
  }
  else if ( PlayersNames [ CPN * 12 -12 + 1 ] != 108 ) {
    for ( int c = 0; c < 12; c++ ) {
      if ( PlayersNames [ CPN * 12 -12 + c ] != 32 ) {
        Serial1.print ( char ( PlayersNames [ CPN * 12 -12 + c ] ));
      }
    }
  }
  Serial1.print ( "'s Turn\n" );
  Serial1.print ( "Shot Starts Game" );
  Shots = 0;
  Wins = 0;
  if ( ServoRelayEnabled == 1 ) {
    // Turn on Servo for Servo duration    
    ServoStartTime = millis ();  // Start Shot LED Timer
    digitalWrite( ServoRelayPin, LOW );   // sets the Servo on  
  }
  while ( Shots == 0 ) {
    CheckSensors();
    WaitingForFirstShot = 1;
    //    GameStartedFlag = 1;
  }
  if ( Shots == 1 ) {
    CheckSensors ();
    GameBInProgress = 1;
    WaitingForFirstShot = 0;
  }
  while ( Shots  < NoS ) {
    CheckSensors ();
  }
  if ( Shots  == NoS ) {    
    GameBInProgress = 0;
    GameOver = 1;
    // Verify if LEDs or bell are still in display cycle and turn off
    if ( ShotLEDEnabled == 1 ) { // 1 = Enabled    
      digitalWrite( ShotLEDPin, LOW );
      // sets the Shot LED off
    }
    if ( WinLEDEnabled == 1 ) { // 1 = Enabled    
      digitalWrite( WinLEDPin, LOW );
      // sets the Win LED off
    }
    /*
    if ( BellRelayEnabled == 1 ) { // 1 = Enabled    
     digitalWrite( BellRelayPin, HIGH );
     // sets the Bell Relay off
     }
     */
    OldTime = millis(); // Sets Counter time back to zero
    // Record Score Counter Game
    int SCORE = Wins * BallsWonPerJackpot - Shots; // Compute Score
    if ( SCORE > 9999 ) {
      SCORE = 9999;
    }
    RecordScore ( GameSelection, SCORE, CPN, NoP, NoS );
    if ( CPN <= NoP ) {
      DisplayScreen ( 2, 4, GameSelection, SCORE, CPN, NoS );
      // Display Player Game Score for 4 Seconds * Delay Multiplier in Function Display Screen
    }
    Shots = 0; // Reset Shots Variable
    Wins = 0;  // Reset Wins Variable
    if ( CPN == NoP ) { // CPN = Current Player Number = NoP = Number of Players - RESET
      // All Players have played this game
      DisplayScreen ( 1, 5, GameSelection, SCORE, CPN, NoS );
      // Display Final Game Scores for 5 Seconds * Delay Multiplier in Function Display Screen
      PreviousGame = 2;
      PreviousGameNumberOfPlayers = NoP;
      PreviousBGameShotNumber = NoS;
      NumberOfPlayers = 0;
      GameSelection = 0;       // Reset Variable
      ShowMenu = 1;            // Reset Variable
      CurrentPlayerNumber = 0; // Reset Variable
    }
  }
  return;
}

/*
[----------------------------------------------------------------------END of Function CounterGame]
 
 [----------------------------------------------------------------START of Function ValidateDigits]
 */

void ValidateDigits (int VD1, int VD2 ) {
  // VD1 = Default Value #, VD2 = Number of Digits
  // Local Variables ( keyi, DigitFirstDigit, 
  // DigitSecondDigit, DigitThirdDigit, DigitFourthDigit
  // LengthSelectionProcess )
  // Passed Values ( VD1, VD2 )

  int DigitFirstDigit = -1;
  int DigitSecondDigit = -1;
  int DigitThirdDigit = -1;
  int DigitFourthDigit = -1;
  char* DisplayValues[]= {
    "Shot LED Duration\n<= 4 Digits in ms.\n#=Accept C=Clear\n",
    "Win LED Duration\n<= 4 Digits in ms.\n#=Accept C=Clear\n",
    "Bell Duration\n<= 4 Digits in ms.\n#=Accept C=Clear\n",
    "Balls per Jackpot\n<= 2 Digits\n#=Accept C=Clear\n",
    "Delay Multiplier\n<= 2 Digits\n#=Accept C=Clear\n",
    "Servo Duration\n<= 4 Digits in ms.\n#=Accept C=Clear\n",
  };
  int DV = 0;
  int LengthSelectionProcess = 0;
  // 0 = Started , 1 = In Process, 2 = Selected, 3 = Confirmed

  if ( VD1 == 11 ) {
    DV = 0;
  }
  else if ( VD1 == 12 ) { 
    DV = 1;
  }
  else if ( VD1 == 13 ) { 
    DV = 2;
  }
  else if ( VD1 == 14 ) { 
    DV = 3;    
  }
  else if ( VD1 == 36 ) { 
    DV = 4;    
  }
  else if ( VD1 == 51 ) {
    DV = 5;
  }
  if ( LengthSelectionProcess == 0 ) { // 0 = Started
    Serial1.print( '\f' ); // \f = Clear screen
    Serial1.print ( DisplayValues[ DV ] );  // display1 varies
    LengthSelectionProcess = 1; // 1 = In Process
  }
  while (LengthSelectionProcess != 3 ) {
    char keyi = keypad.getKey();
    if  ( keyi != NO_KEY ) {
      switch ( keyi ){
      case '#': // Accept
        //Check for Valid Number of Digits
        if ( DigitFirstDigit == -1 ) {
          // Don't Change Default and return
          return;
        }
        LengthSelectionProcess = 2; // 2 = Selected
        break;
      case 'C': // Cancel
        DigitFirstDigit = -1;
        DigitSecondDigit = -1;
        DigitThirdDigit = -1;
        DigitFourthDigit = -1;
        Serial1.print( '\f' ); // \f = Clear screen
        Serial1.print ( DisplayValues[ DV ] ); // display1 varies
        LengthSelectionProcess = 1;  // 1 = In Process
        break;
      case 'B':
        break;
      case 'D':
        break;
      case '*':
        break;
      case 'A':
        break;
      default: // If key is not A or C; B,D,*,#
        if ( DigitFirstDigit == -1 ) {
          if ( keyi == '0' ) {
            DigitFirstDigit = -1;
            DigitSecondDigit = -1;
            DigitThirdDigit = -1;
            DigitFourthDigit = -1;
            Serial1.print( '\f' ); // \f = Clear screen
            Serial1.print ( DisplayValues[ DV ] ); // display1 varies
            LengthSelectionProcess = 0;  // 1 = In Process
            break;
          }
        }
        if ( DigitFirstDigit == -1 && keyi != '0' ) {
          DigitFirstDigit = keyi ; // Don't convert char to dec
          Serial1.print ( DigitFirstDigit - 48 ); // Convert char to dec
        } 
        else if ( DigitFirstDigit != -1 && DigitSecondDigit == -1 ) {
          DigitSecondDigit = keyi ;  // Don't convert char to dec
          Serial1.print ( DigitSecondDigit - 48 ); // Convert char to dec
          /*if ( VD1 == 12 ) {
            LengthSelectionProcess = 2; // 2 = Selected
          }
         */ 
        }
        else if ( DigitFirstDigit != -1 && DigitSecondDigit != -1 && DigitThirdDigit == -1 ) {
          DigitThirdDigit = keyi ;   // Don't convert char to dec
          Serial1.print ( DigitThirdDigit - 48 ); // Convert char to dec
        }
        else if ( DigitFirstDigit != -1 && DigitSecondDigit != -1 && DigitThirdDigit != -1 ) {
          DigitFourthDigit = keyi ;   // Don't convert char to dec
          Serial1.print ( DigitFourthDigit - 48 ); // Convert char to dec
          LengthSelectionProcess = 2; // 2 = Selected
        }
      }
    }
    if (LengthSelectionProcess == 2) { //Digit Selections completed
      if ( VD2 == 4 ) {
        EEPROM.write ( EEPROMLocations [ VD1 ], DigitFirstDigit  );
        if ( DigitSecondDigit != -1 ) {
          EEPROM.write (( EEPROMLocations [ VD1 ] + 1 ), DigitSecondDigit  );
        }
        else if ( DigitSecondDigit == -1 ) { // Over write with " " = ASCII 32
          EEPROM.write (( EEPROMLocations [ VD1 ] + 1 ), 32  );
        }
        if ( DigitThirdDigit != -1 ) {
          EEPROM.write (( EEPROMLocations [ VD1 ] + 2 ), DigitThirdDigit  );
        }
        else if ( DigitThirdDigit == -1 ) { // Over write with " " = ASCII 32
          EEPROM.write (( EEPROMLocations [ VD1 ] + 2 ), 32  );
        }
        if ( DigitFourthDigit != -1 ) {
          EEPROM.write (( EEPROMLocations [ VD1 ] + 3 ), DigitFourthDigit  );
        }
        else if ( DigitFourthDigit == -1 ) { // Over write with " " = ASCII 32
          EEPROM.write (( EEPROMLocations [ VD1 ] + 3 ), 32  );
        }
      }
      else if ( VD2 == 2 ) {
        EEPROM.write ( EEPROMLocations [ VD1 ], DigitFirstDigit  );
        if ( DigitSecondDigit != -1 ) {
          EEPROM.write (( EEPROMLocations [ VD1 ] + 1 ), DigitSecondDigit  );
        }
        else if ( DigitSecondDigit == -1 ) { // Over write with " " = ASCII 32
          EEPROM.write (( EEPROMLocations [ VD1 ] + 1 ), 32  );
        }
      }
      LengthSelectionProcess = 3; // 3 = Confirmed 
    }
  }
  Serial1.print ( "\f" );
  return;
}

/*
[-------------------------------------------------------------------END of Function ValidateDigits]
 
 [---------------------------------------------------------------------START of Function ReadValue]
 */

int ReadValue ( int RV ) {
  // Local Variables ( RVV1, RVV2, RVV3, returndigit1, returndigit2, returndigit3,
  // returndigit4, returnresult )

  // Passed Variable RV = Value to Read

  int RVV1 = 0;  // Determine number of digits in ReadValue ( RVV1 possible values = 1, 2, 3, 4 )
  for ( int RVV2 = 0; RVV2 < EEPROMLength [ RV ]; RVV2++ ) {
    int RVV3 = EEPROM.read ( ( EEPROMLocations [ RV ] ) + RVV2 );
    if ( RVV3 != 32 ) {
      RVV1++;
    }
  }
  if ( RVV1 == 1 ) {
    int returndigit1 = EEPROM.read ( EEPROMLocations [ RV ] );
    return ( returndigit1 - 48 ); // Convert from ASCII to dec
  }
  else if ( RVV1 == 2 ) {
    int returndigit1 = EEPROM.read ( EEPROMLocations [ RV ] );
    int returndigit2 = EEPROM.read (( EEPROMLocations [ RV ]) + 1 );
    int returnresult = ((( returndigit1 - 48) * 10) + ( returndigit2 - 48 ));
    return ( returnresult ); // return result value
  }
  else if ( RVV1 == 3 ) {
    int returndigit1 = EEPROM.read ( EEPROMLocations [ RV ] );
    int returndigit2 = EEPROM.read (( EEPROMLocations [ RV ]) + 1 );
    int returndigit3 = EEPROM.read (( EEPROMLocations [ RV ]) + 2 );
    int returnresult = ((( returndigit1 - 48 ) * 100) + (( returndigit2 - 48 ) * 10)
      + ( returndigit3 - 48 ));
    return ( returnresult );
  }  
  else if ( RVV1 == 4 ) {
    int returndigit1 = EEPROM.read ( EEPROMLocations [ RV ] );
    int returndigit2 = EEPROM.read (( EEPROMLocations [ RV ]) + 1 );
    int returndigit3 = EEPROM.read (( EEPROMLocations [ RV ]) + 2 );
    int returndigit4 = EEPROM.read (( EEPROMLocations [ RV ]) + 3 );
    int returnresult = ((( returndigit1 - 48 ) * 1000) + (( returndigit2 - 48 ) * 100)
      + (( returndigit3 - 48)* 10 ) + ( returndigit4 - 48 ));
    return ( returnresult );
  }
}

/*
[------------------------------------------------------------------------END of Function ReadValue]
 
 [-------------------------------------------------------------------START of Function RecordScore]
 */

void RecordScore ( byte GS, int SC, byte CPN, byte NoP, int Limit ) {
  // GS = GameSelection
  // SC = SCORE
  // CPN = CurrentPlayerNumber
  // NoP = NumberOfPlayers
  // if GameSelection = 1; Limit = A Game Time Length ( in seconds NOT in milliseconds )
  // if GameSelection = 2; Limit = B Game Shot Number
  // Local Variables ( HighScore1 through HighScore5, HighScore5Location,
  // HighScore5PlayerLocation, ScoreDigit1 through ScoreDigit4, YYY, x, XXX )
  // Passed Values ( GS, SC, CPN, NoP, Limit )

  // Score Portion of Function

  // These values are stored in PlayerScoreArray after each Players' turn
  PlayerScoreArray [ ( 1 + (( CPN - 1 ) * 4 )) ] = Shots;
  // Write Player's Shots in PlayerScoreArray
  PlayerScoreArray [ ( 2 + (( CPN - 1 ) * 4 )) ] = Wins;
  // Write Player's Wins in PlayerScoreArray
  PlayerScoreArray [ ( 3 + (( CPN - 1 ) * 4 )) ] = SC;
  // Write Player's SCORE in PlayerScoreArray

  // Rank Portion of Function
  if ( CPN == NoP ) { // Wait until all players have taken their turn  
    // wait until CurrentPlayerNumber is equal to NumberOfPlayers to determine rankings
    ScoreRankArray[0] = PlayerScoreArray [3];  // Store Player 1 SCORE in ScoreRankArray unsorted
    ScoreRankArray[1] = PlayerScoreArray [7];  // Store Player 2 SCORE in ScoreRankArray unsorted
    ScoreRankArray[2] = PlayerScoreArray [11]; // Store Player 3 SCORE in ScoreRankArray unsorted
    ScoreRankArray[3] = PlayerScoreArray [15]; // Store Player 4 SCORE in ScoreRankArray unsorted
    for ( int x = 0; x < NoP-1; x++ ) { // Bubble Sort ScoreRankArray
      for ( int y = 0; y < NoP - x-1; y++ ) {
        if ( ScoreRankArray [ y ] < ScoreRankArray [ y + 1 ] ) {
          int tempz = ScoreRankArray [ y + 1 ];
          int tempz2 = ScoreRankArray [ y + 5 ];
          ScoreRankArray [ y + 1 ] = ScoreRankArray [ y ];
          ScoreRankArray [ y + 5 ] = ScoreRankArray [ y + 4 ];
          ScoreRankArray [ y ] = tempz;
          ScoreRankArray [ y + 4 ] = tempz2;
        }
      }
    }

    // Result of Bubble Sort = High Score 1, High Score 2, High Score 3, High Score 4,
    // 1st Place Player Number through 4th Place Player Number
    // Ties are allowed

    // Store Game Information regardless of Number of Players
    // Store Previous Game Number ( 1 = Timer Game; 2 = Counter Game )
    // in dec ( Not ASCII ) in location 37
    EEPROM.write ( EEPROMLocations [ 37 ], GS );

    // Store Previous Game Number of Players
    // in dec ( Not ASCII ) in location 38
    EEPROM.write ( EEPROMLocations [ 38 ], NoP );

    // Store Results of Bubble Sort in EEPROM
    // Break Score into digits for each player
    // Extract Score Digits from each SCORE from highest to lowest SCORE   

    for ( int m = 0; m < NoP; m++ ) {    
      int PlaceScore = ScoreRankArray [ m ];
      int ScoreDigit1 = -1;
      int ScoreDigit2 = -1;
      int ScoreDigit3 = -1;
      int ScoreDigit4 = -1;
      if ( PlaceScore > 999 ) { // PlaceScore has four digits
        ScoreDigit1 = int ( PlaceScore / 1000 );
        ScoreDigit2 = int ((( PlaceScore - ( ScoreDigit1 * 1000 )) / 100 ));
        ScoreDigit3 = int (( PlaceScore - (( ScoreDigit1 * 1000 ) + ( ScoreDigit2 * 100 ))) / 10);
        ScoreDigit4 = int ( PlaceScore - (( ScoreDigit1 * 1000 ) + ( ScoreDigit2 * 100 ) +
          ( ScoreDigit3 * 10 )));
        ScoreDigit1 = ScoreDigit1 + 48; // Convert to ASCII
        ScoreDigit2 = ScoreDigit2 + 48; // Convert to ASCII
        ScoreDigit3 = ScoreDigit3 + 48; // Convert to ASCII
        ScoreDigit4 = ScoreDigit4 + 48; // Convert to ASCII
      }
      else if ( PlaceScore < 1000 && PlaceScore > 99 ) { // PlaceScore has three digits
        ScoreDigit1 = int ( PlaceScore / 100 );
        ScoreDigit2 = int ((( PlaceScore - ( ScoreDigit1 * 100 )) / 10 ));
        ScoreDigit3 = int (( PlaceScore - (( ScoreDigit1 * 100 ) + ( ScoreDigit2 * 10 ))));
        ScoreDigit1 = ScoreDigit1 + 48; // Convert to ASCII
        ScoreDigit2 = ScoreDigit2 + 48; // Convert to ASCII
        ScoreDigit3 = ScoreDigit3 + 48; // Convert to ASCII
        ScoreDigit4 = 32; // " " = ASCII in Space
      }
      else if ( PlaceScore < 100 && PlaceScore > 9 ) { // SCORE has two digits
        ScoreDigit1 = int ( PlaceScore / 10 );
        ScoreDigit2 = int ( PlaceScore - ( ScoreDigit1 * 10 ));
        ScoreDigit1 = ScoreDigit1 + 48; // Convert to ASCII
        ScoreDigit2 = ScoreDigit2 + 48; // Convert to ASCII
        ScoreDigit3 = 32; // " " = ASCII in Space
        ScoreDigit4 = 32; // " " = ASCII in Space
      }
      else if ( PlaceScore < 10 && PlaceScore > 0 ) { // SCORE has one digit
        ScoreDigit1 = PlaceScore + 48; // Convert to ASCII
        ScoreDigit2 = 32; // " " = ASCII in Space
        ScoreDigit3 = 32; // " " = ASCII in Space
        ScoreDigit4 = 32; // " " = ASCII in Space
      }
      else if ( PlaceScore <= 0 ) { // SCORE is negative and is Zero
        ScoreDigit1 = 48; // Convert to ASCII
        ScoreDigit2 = 32; // " " = ASCII in Space
        ScoreDigit3 = 32; // " " = ASCII in Space
        ScoreDigit4 = 32; // " " = ASCII in Space
      }    
      ScoreRankArray2 [ 0 + ( m * 4 ) ] = ScoreDigit1;
      ScoreRankArray2 [ 1 + ( m * 4 ) ] = ScoreDigit2;
      ScoreRankArray2 [ 2 + ( m * 4 ) ] = ScoreDigit3;
      ScoreRankArray2 [ 3 + ( m * 4 ) ] = ScoreDigit4;
      // Store Previous Game Number SCORE 1 through SCORE 4, digits 1 through 4
      // ( in  ASCII ) in locations 43 through 46    
      for ( int y = 0; y < EEPROMLength [ 43 + m ]; y++ ) {
        EEPROM.write (( EEPROMLocations [ 43 + m ] + y ), ScoreRankArray2 [ m * 4 + y ] );
      }
      // Store Previous Game Number Player Name for SCORE 1
      // through Player Name for SCORE 4
      // ( in  ASCII ) in locations 39 through 42    
      for ( int y = 0; y < EEPROMLength [ 39 + m ]; y++ ) {
        EEPROM.write (( EEPROMLocations [ 39 + m ] + y ), 
        PlayersNames [ ScoreRankArray [ 4 + m ]  * 12 - 12 + y  ] );
      }
    } 
  }

  // High Score Portion of Function
  // This portion is executed after each game is played
  int HighScore1 = 0;
  int HighScore2 = 0;
  int HighScore3 = 0;
  int HighScore4 = 0;
  int HighScore5 = 0;

  // int HighScore1Location = 0;
  int HighScore5Location = 0;
  // int HighScore1PlayerLocation = 0;
  int HighScore5PlayerLocation = 0;

  int ScoreDigit1 = -1;
  int ScoreDigit2 = -1;
  int ScoreDigit3 = -1;
  int ScoreDigit4 = -1;

  int YYY = 0; // Used to write high scores

  // Extract Score Digits from SCORE ( SC )
  if ( SC > 999 ) { // SCORE has four digits
    ScoreDigit1 = int ( SC / 1000 );
    ScoreDigit2 = int ((( SC - ( ScoreDigit1 * 1000 )) / 100 ));
    ScoreDigit3 = int (( SC - (( ScoreDigit1 * 1000 ) + ( ScoreDigit2 * 100 ))) / 10);
    ScoreDigit4 = int ( SC - (( ScoreDigit1 * 1000 ) + ( ScoreDigit2 * 100 ) +
      ( ScoreDigit3 * 10 )));

    ScoreDigit1 = ScoreDigit1 + 48; // Convert to ASCII
    ScoreDigit2 = ScoreDigit2 + 48; // Convert to ASCII
    ScoreDigit3 = ScoreDigit3 + 48; // Convert to ASCII
    ScoreDigit4 = ScoreDigit4 + 48; // Convert to ASCII
  }
  else if ( SC < 1000 ) { // SCORE has three digits or it is not a new high score
    ScoreDigit1 = int ( SC / 100 );
    ScoreDigit2 = int ((( SC - ( ScoreDigit1 * 100 )) / 10 ));
    ScoreDigit3 = int (( SC - (( ScoreDigit1 * 100 ) + ( ScoreDigit2 * 10 ))));

    ScoreDigit1 = ScoreDigit1 + 48; // Convert to ASCII
    ScoreDigit2 = ScoreDigit2 + 48; // Convert to ASCII
    ScoreDigit3 = ScoreDigit3 + 48; // Convert to ASCII
    ScoreDigit4 = 32; // " " = ASCII in Space
  }

  if ( GS == 1 ) { // Timer Game
    if ( Limit != ReadValue( 5 )) { // Don't record High Score
      // Limit is NOT equal default Game A Time Length
      return;
    }
  }
  else if ( GS == 2 ) { // Counter Game
    if ( Limit != ReadValue( 7 )) { // Don't record High Score
      // Limit is NOT equal default Game B Shot Number
      return;
    }
  }
  if ( GS == 1 ) { // Timer Game
    HighScore1 = ReadValue( 20 );
    HighScore2 = ReadValue( 21 );
    HighScore3 = ReadValue( 22 );
    HighScore4 = ReadValue( 23 );
    HighScore5 = ReadValue( 24 );
    HighScore5Location = 24;
    HighScore5PlayerLocation = 19;
  }
  else if ( GS == 2 ) { // Counter Game
    HighScore1 = ReadValue( 25 );
    HighScore2 = ReadValue( 26 );
    HighScore3 = ReadValue( 27 );
    HighScore4 = ReadValue( 28 );
    HighScore5 = ReadValue( 29 );
    HighScore5Location = 29;
    HighScore5PlayerLocation = 34;
  }
  if ( SC <= HighScore5 ) { // Not a new High Score - don't record
    return;
  }
  else if ( SC > HighScore5 && SC <= HighScore4 ) {
    // Replace High Score 5
    // New High Score 5 equals SCORE; similarly New Player 5 = Current Player
    // Write High Score 5 - HighScore5Location  
    EEPROM.write ( EEPROMLocations [ HighScore5Location ], ScoreDigit1 );
    EEPROM.write ( ( EEPROMLocations [ HighScore5Location ] + 1 ), ScoreDigit2 );
    EEPROM.write ( ( EEPROMLocations [ HighScore5Location ] + 2 ), ScoreDigit3 );
    EEPROM.write ( ( EEPROMLocations [ HighScore5Location ] + 3 ), ScoreDigit4 );
    // Write High Score 5 Player Name    
    for ( int x = 0; x < EEPROMLength [ HighScore5PlayerLocation ]; x++ ) {
      byte EEPROMNameValue = PlayersNames [ CPN * 12 - 12 + x ];
      // Use PlayersNames Array Value
      EEPROM.write ( ( EEPROMLocations [ HighScore5PlayerLocation ] + x ), EEPROMNameValue );
    }   
    return;
  }
  else if ( SC > HighScore4 && SC <= HighScore3 ) {
    // Replace High Scores 5, 4
    // New High Score 5 equals Old High Score 4; similarly New Player 5 = Old Player 4
    // New High Score 4 equals SCORE; similarly New Player 4 = Current Player  
    YYY = 1;
  }
  else if ( SC > HighScore3 && SC <= HighScore2 ) {
    // Replace High Scores 5, 4, 3
    // New High Score 5 equals Old High Score 4; similarly New Player 5 = Old Player 4
    // New High Score 4 equals Old High Score 3; similarly New Player 4 = Old Player 3
    // New High Score 3 equals SCORE; similarly New Player 3 = Current Player     
    YYY = 2;
  }
  else if ( SC > HighScore2 && SC <= HighScore1 ) {
    // Replace High Scores 5, 4, 3, 2 
    // New High Score 5 equals Old High Score 4; similarly New Player 5 = Old Player 4
    // New High Score 4 equals Old High Score 3; similarly New Player 4 = Old Player 3
    // New High Score 3 equals Old High Score 2; similarly New Player 3 = Old Player 2
    // New High Score 2 equals SCORE; similarly New Player 2 = Current Player 
    YYY = 3;
  }
  else if ( SC > HighScore1 ) {
    YYY = 4;
  }
  // Replace High Scores 5, 4, 3, 2, 1
  // New High Score 5 equals Old High Score 4; similarly New Player 5 = Old Player 4
  // New High Score 4 equals Old High Score 3; similarly New Player 4 = Old Player 3
  // New High Score 3 equals Old High Score 2; similarly New Player 3 = Old Player 2
  // New High Score 2 equals Old High Score 1; similarly New Player 2 = Old Player 1
  // New High Score 1 equals SCORE; similarly New Player 1 = Current Player

  // Replace High Score 5 with High Score 4 etc. replace High Score 2 with High Score 1
  for ( int XXX = 0; XXX < YYY; XXX++ ) {
    for ( int x = 0; x < EEPROMLength [ ( HighScore5Location - XXX ) ]; x++ ) {
      byte OldEEPROMValue = EEPROM.read (( EEPROMLocations [ ( HighScore5Location - 1 - XXX ) ] + x ));    
      EEPROM.write ( ( EEPROMLocations [ ( HighScore5Location - XXX ) ] + x ), OldEEPROMValue );
    }
    for ( int x = 0; x < EEPROMLength [ ( HighScore5PlayerLocation - XXX ) ]; x++ ) {
      byte OldEEPROMValue = EEPROM.read (( EEPROMLocations [ ( HighScore5PlayerLocation - 1 - XXX ) ] + x ));    
      EEPROM.write ( ( EEPROMLocations [ ( HighScore5PlayerLocation - XXX ) ] + x ), OldEEPROMValue );
    }  
  }
  // Write High Score 1 - ( HighScore5Location - 4 ) = HighScore1Location   
  EEPROM.write ( EEPROMLocations [ ( HighScore5Location - YYY ) ], ScoreDigit1 );
  EEPROM.write ( ( EEPROMLocations [ ( HighScore5Location - YYY ) ] + 1 ), ScoreDigit2 );
  EEPROM.write ( ( EEPROMLocations [ ( HighScore5Location - YYY ) ] + 2 ), ScoreDigit3 );
  EEPROM.write ( ( EEPROMLocations [ ( HighScore5Location - YYY ) ] + 3 ), ScoreDigit4 );
  // Write High Score 1 Player Name    
  for ( int x = 0; x < EEPROMLength [ ( HighScore5PlayerLocation - YYY ) ]; x++ ) {
    byte EEPROMNameValue = PlayersNames [ CPN * 12 - 12 + x ]; // Use PlayersNames Array Value    
    EEPROM.write ( ( EEPROMLocations [ ( HighScore5PlayerLocation - YYY ) ] + x ), EEPROMNameValue );
  }
  return;
}

/*
[----------------------------------------------------------------------END of Function RecordScore]
 
 [-----------------------------------------------------------------START of Function DisplayScreen]
 */
void DisplayScreen ( int DSNum, byte DSec, byte GS, int SCORE, int CPN, int LIMIT ) {
  // DSNum = 1 Show Final Scores
  // DSNum = 2 Show Player Score
  // DSNum = 3 Show High Scores 
  // DSec = Delay Seconds
  // GS = Game Selection; 1 = Timer Game; 2 = Counter Game
  // SCORE = SCORE
  // CPN = Current Player Number
  // LIMIT = AGameTimeLength ( in Seconds ) = BGameNumberOfShots

  if ( DSNum == 1 ) {
    Serial1.print( '\f' );        // \f = Clear screen
    for ( int a = 0; a < EEPROM.read( EEPROMLocations[ 38 ] ); a++ ) {    
      Serial1.print ( a + 1 );
      Serial1.print ( " " ); // One Space
      for ( int x = 0; x < EEPROMLength [ 39 + a ] ; x++ ) {
        if ( EEPROM.read( EEPROMLocations[ 39 + a ] + x ) != 255 ) {
          Serial1.print ( char ( EEPROM.read( EEPROMLocations[ 39 + a ] + x )));
        }      
      }
      Serial1.print ( " " ); // One Space
      for ( int x = 0; x < EEPROMLength [ 43 + a ] ; x++ ) {
        if ( EEPROM.read( EEPROMLocations[ 43 + a ] + x ) != 255 ) {
          Serial1.print ( char ( EEPROM.read( EEPROMLocations[ 43 + a ] + x )));
        }      
      }
      if ( a < 3 ) {
        Serial1.print ( '\n' );
      }
    }
    delay ( 600 * DelayMultiplierWait );
    return;
  }
  else if ( DSNum == 2 ) {
    Serial1.print( '\f' );        // \f = Clear screen
    if ( GS == 1 ) {
      Serial1.print ( "     TIMER GAME\n" );
    }
    else if ( GS == 2 ) {
      Serial1.print ( "    COUNTER GAME\n" );
    }
    Serial1.print ( "   SCORE     " ); // Three spaces SCORE Five Spaces
    Serial1.print ( SCORE );
    Serial1.print ( "\n    " ); // Four Spaces
    for ( int c = 0; c < 12; c++ ) {
      if ( PlayersNames [ CPN * 12 -12 + c ] != 255  ) {
        Serial1.print ( char ( PlayersNames [ CPN * 12 -12 + c ] ));
      }      
    }   
    Serial1.print ( "\n    " ); // Four Spaces
    Serial1.print ( LIMIT );
    if ( GS == 1 ) {
      Serial1.print ( " Seconds" );
    }
    else if ( GS == 2 ) {
      Serial1.print ( " Shots" );
    }
    delay ( 600 * DelayMultiplierWait );
    return;
  }
  else if ( DSNum == 3 ) {
    for ( int a = 0; a < 5; a++ ) { // Timer Game
      Serial1.print ( '\f' ); // Clear Screen
      Serial1.print ( "    Timer Game\n" ); // Four Spaces
      Serial1.print ( " High Score " ); // One Space
      Serial1.print ( a + 1 );
      Serial1.print ( " " ); // One Space
      for ( int x = 0; x < EEPROMLength [ 20 + a ] ; x++ ) { // Score
        if ( EEPROM.read( EEPROMLocations[ 20 + a ] + x ) != 255 ) {
          Serial1.print ( char ( EEPROM.read( EEPROMLocations[ 20 + a ] + x )));
        }    
      }
      Serial1.print ( "\n    " ); // Four Spaces
      for ( int x = 0; x < EEPROMLength [ 15 + a ] ; x++ ) { // Name
        if ( EEPROM.read( EEPROMLocations[ 15 + a ] + x ) != 255 ) {
          Serial1.print ( char ( EEPROM.read( EEPROMLocations[ 15 + a ] + x )));
        }      
      } 
      delay ( 600 * DelayMultiplierWait );
      Serial1.print ( '\f' ); // Clear Screen
      Serial1.print ( "   Counter Game\n" ); // Four Spaces
      Serial1.print ( " High Score " ); // One Space
      Serial1.print ( a + 1 );
      Serial1.print ( " " ); // One Space
      for ( int x = 0; x < EEPROMLength [ 25 + a ] ; x++ ) { // Score
        if ( EEPROM.read( EEPROMLocations[ 25 + a ] + x ) != 255 ) {
          Serial1.print ( char ( EEPROM.read( EEPROMLocations[ 25 + a ] + x )));
        }    
      }
      Serial1.print ( "\n    " ); // Four Spaces
      for ( int x = 0; x < EEPROMLength [ 30 + a ] ; x++ ) { // Name
        if ( EEPROM.read( EEPROMLocations[ 30 + a ] + x ) != 255 ) {
          Serial1.print ( char ( EEPROM.read( EEPROMLocations[ 30 + a ] + x )));
        }      
      }         
      delay ( 600 * DelayMultiplierWait );
    }
  }
  return;
}

/*
[--------------------------------------------------------------------END of Function DisplayScreen]
 
 [-------------------------------------------------------------------START of Function ShowOptions]
 */

void ShowOptions () {
  // Local Variables ( WaitForConfirm1, getkey )

  Serial1.print ( "\f" ); // Clear Screen
  Serial1.print ( "D = Defaults\n" );
  Serial1.print ( "C = Previous Score\n" );
  Serial1.print ( "A = High Scores\n" );
  Serial1.print ( "# = Return" );  

  byte WaitForConfirm1 = 0;
  while ( WaitForConfirm1 == 0 ) {
    char getkey = GetKeypadInput ( 1 );
    if ( getkey == 'D' ) { // Call ChangeDefaults Function
      ChangeDefaults ();      
      WaitForConfirm1 = 1;
    }
    else if ( getkey == 'C' ) { // Show Previous Score
      DisplayScreen ( 1, 10, 0, 0, 0, 0 );
      WaitForConfirm1 = 1;
    }
    else if ( getkey == 'A' ) { // Show High Scores      
      DisplayScreen ( 3, 5, 0, 0, 0, 0 );
      WaitForConfirm1 = 1;
    }
    else if ( getkey == '#' ) { // Return      
      WaitForConfirm1 = 1;
      return;
    }
  }
  return;
}
/*
[----------------------------------------------------------------------END of Function ShowOptions]
 
 [--------------------------------------------------------------------START of Interrupt Functions]
 */
void ShotwasSensed () {
  // Interrupt 0 Function
  // Local Values - none
  Shots ++;
  return;
}

void WinwasSensed () {
  // Interrupt 1 Function
  // Local Values - none
  Wins ++;
  return;
}
/*
[-----------------------------------------------------------------------END of Interrupt Functions]
 
 [------------------------------------------------------------------------------------------------]
 */

















